@@ -10,6 +10,7 @@
 #include "art-enum.h"
 #include "attack.h"
 #include "chardump.h"
+#include "delay.h"
 #include "directn.h"
 #include "env.h"
 #include "fight.h" // apply_chunked_ac
@@ -187,7 +188,7 @@ bool actor::can_sleep(bool holi_only) const
     return true;
 }
 
-void actor::shield_block_succeeded()
+void actor::shield_block_succeeded(actor *attacker)
 {
     item_def *sh = shield();
     const unrandart_entry *unrand_entry;
@@ -200,16 +201,23 @@ void actor::shield_block_succeeded()
         && (unrand_entry = get_unrand_entry(sh->unrand_idx))
         && unrand_entry->melee_effects)
     {
-        unrand_entry->melee_effects(sh, this, nullptr, false, 0);
+        unrand_entry->melee_effects(sh, this, attacker, false, 0);
     }
 }
 
+/// How many levels of penalties does this actor have from inaccuracy-conferring effects?
 int actor::inaccuracy() const
 {
     const item_def *amu = slot_item(EQ_AMULET);
     return amu && is_unrandom_artefact(*amu, UNRAND_AIR);
 }
 
+/// How great are the penalties to this actor's to-hit from any inaccuracy effects they have?
+int actor::inaccuracy_penalty() const
+{
+    return inaccuracy() * 5;
+}
+
 bool actor::res_corr(bool /*allow_random*/, bool temp) const
 {
     return temp && (wearing(EQ_RINGS, RING_RESIST_CORROSION)
@@ -280,10 +288,14 @@ bool actor::reflection(bool items) const
             || wearing_ego(EQ_ALL_ARMOUR, SPARM_REFLECTION));
 }
 
-bool actor::extra_harm(bool items) const
+int actor::extra_harm(bool items) const
 {
-    return items &&
-           (wearing_ego(EQ_CLOAK, SPARM_HARM) || scan_artefacts(ARTP_HARM));
+    if (!items)
+        return 0;
+
+    int harm = wearing_ego(EQ_CLOAK, SPARM_HARM) + scan_artefacts(ARTP_HARM);
+
+    return harm > 2 ? 2 : harm;
 }
 
 bool actor::rmut_from_item() const
@@ -330,7 +342,8 @@ bool actor::rampaging(bool items) const
     return items &&
            (wearing_ego(EQ_ALL_ARMOUR, SPARM_RAMPAGING)
             || scan_artefacts(ARTP_RAMPAGING)
-            || is_player() && player_equip_unrand(UNRAND_SEVEN_LEAGUE_BOOTS));
+            || is_player() && (player_equip_unrand(UNRAND_SEVEN_LEAGUE_BOOTS)
+                               || you.has_mutation(MUT_ROLLPAGE)));
 }
 
 int actor::apply_ac(int damage, int max_damage, ac_type ac_rule, bool for_real) const
@@ -793,8 +806,12 @@ void actor::constriction_damage_defender(actor &defender, int duration)
          defender.name(DESC_PLAIN, true).c_str(),
          basedam, durdam, acdam, timescale_dam, infdam);
 
-    if (defender.is_monster() && defender.as_monster()->hit_points < 1)
+    if (defender.is_monster()
+        && defender.type != MONS_NO_MONSTER // already dead and reset
+        && defender.as_monster()->hit_points < 1)
+    {
         monster_die(*defender.as_monster(), this);
+    }
 }
 
 // Deal damage over time
@@ -906,7 +923,8 @@ bool actor::torpor_slowed() const
         const monster *mons = *ri;
         if (mons && mons->type == MONS_TORPOR_SNAIL
             && !is_sanctuary(mons->pos())
-            && !mons_aligned(mons, this))
+            && !mons_aligned(mons, this)
+            && !mons->props.exists(KIKU_WRETCH_KEY))
         {
             return true;
         }
@@ -957,7 +975,7 @@ void actor::collide(coord_def newpos, const actor *agent, int pow)
     if (is_monster() && !god_prot)
         behaviour_event(as_monster(), ME_WHACK, agent);
 
-    dice_def damage(2, 1 + pow / 10);
+    dice_def damage(2, 1 + div_rand_round(pow, 10));
 
     if (other && other->alive())
     {
@@ -1031,6 +1049,82 @@ void actor::collide(coord_def newpos, const actor *agent, int pow)
     }
 }
 
+/**
+ * @brief Attempts to knock this actor away from a specific tile,
+ *        using repeated one-tile knockbacks. Any LOS requirements
+ *        must be checked by the calling function.
+ * @param cause The actor responsible for the knockback.
+ * @param dist How far back to try to push this actor.
+ * @param pow Determines damage done to us if we hit something. If -1, don't do damage.
+ * @param source_name The name of the thing that's pushing this actor.
+ * @returns True if this actor is moved from their initial position; false otherwise.
+ */
+
+bool actor::knockback(const actor &cause, int dist, int pow, string source_name)
+{
+    if (is_stationary() || resists_dislodge("being knocked back"))
+        return false;
+
+    const coord_def source = cause.pos();
+    const coord_def oldpos = pos();
+
+    if (source == oldpos)
+        return false;
+
+    ray_def ray;
+    fallback_ray(source, oldpos, ray);
+    while (ray.pos() != oldpos)
+        ray.advance();
+
+    coord_def newpos = oldpos;
+    for (int dist_travelled = 0; dist_travelled < dist; ++dist_travelled)
+    {
+        const ray_def oldray(ray);
+
+        ray.advance();
+
+        newpos = ray.pos();
+        if (newpos == oldray.pos()
+            || !in_bounds(newpos)
+            || cell_is_solid(newpos)
+            || actor_at(newpos)
+            || !can_pass_through(newpos)
+            || !is_habitable(newpos))
+        {
+            ray = oldray;
+            break;
+        }
+
+        move_to_pos(newpos);
+        if (is_player())
+            stop_delay(true);
+    }
+
+    if (newpos == oldpos)
+        return false;
+
+    if (you.can_see(*this))
+    {
+        mprf("%s %s knocked back by the %s.",
+             name(DESC_THE).c_str(),
+             conj_verb("are").c_str(),
+             source_name.c_str());
+    }
+
+    if (pow != -1 && pos() != newpos)
+        collide(newpos, &cause, pow);
+
+    // Stun the monster briefly so that it doesn't look as though it wasn't
+    // knocked back at all
+    if (is_monster())
+        as_monster()->speed_increment -= random2(6) + 4;
+
+    apply_location_effects(oldpos, cause.is_player()? KILL_YOU_MISSILE : KILL_MON_MISSILE,
+                                actor_to_death_source(&cause));
+
+    return true;
+}
+
 /// Is this creature despised by the so-called 'good gods'?
 bool actor::evil() const
 {
