@@ -33,16 +33,18 @@
 #include "english.h"       // For apostrophise
 #include "exercise.h"      // For practise_evoking
 #include "fight.h"
+#include "fineff.h"        // For the Storm Queen's Shield
 #include "god-conduct.h"   // did_god_conduct
-#include "mgen-data.h"     // For Sceptre of Asmodeus evoke
+#include "mgen-data.h"     // For Sceptre of Asmodeus
 #include "melee-attack.h"  // For autumn katana
 #include "message.h"
 #include "monster.h"
 #include "mon-death.h"     // For demon axe's SAME_ATTITUDE
-#include "mon-place.h"     // For Sceptre of Asmodeus evoke
+#include "mon-place.h"     // For Sceptre of Asmodeus
 #include "nearby-danger.h" // For Zhor
 #include "output.h"
 #include "player.h"
+#include "player-reacts.h" // For the consecrated labrys
 #include "player-stats.h"
 #include "showsymb.h"      // For Cigotuvi's Embrace
 #include "spl-cast.h"      // For evokes
@@ -303,6 +305,20 @@ static void _POWER_melee_effects(item_def* /*weapon*/, actor* attacker,
 
 ////////////////////////////////////////////////////
 
+static void _HOLY_AXE_world_reacts(item_def *item)
+{
+    const int horror_level = current_horror_level();
+    // Caps at the obsidian axe's base enchant.
+    const int plus = min(horror_level * 3 + 4, 16);
+    if (item->plus == plus)
+        return;
+
+    item->plus = plus;
+    you.wield_change = true;
+}
+
+////////////////////////////////////////////////////
+
 static void _SINGING_SWORD_equip(item_def *item, bool *show_msgs, bool /*unmeld*/)
 {
     bool def_show = true;
@@ -513,10 +529,32 @@ static void _GONG_melee_effects(item_def* /*item*/, actor* wearer,
 
 ///////////////////////////////////////////////////
 
+static void _STORM_QUEEN_melee_effects(item_def* /*item*/, actor* wearer,
+                                       actor* attacker, bool /*dummy*/,
+                                       int /*dam*/)
+{
+    // Discharge does 3d(4 + pow*3/2) damage, so each point of power does
+    // an average of another 9/4 points of retaliation damage (~2).
+    // Let's try 3d7 damage at 1/3 chance. This is broadly comparable to
+    // elec brand - same average damage per trigger, higher trigger chance,
+    // but checks (half) AC - and triggers on block instead of attack :)
+    if (!attacker || !one_chance_in(3)) return;
+    shock_discharge_fineff::schedule(wearer, *attacker,
+                                     wearer->pos(), 3,
+                                     "shield");
+
+}
+
+///////////////////////////////////////////////////
+
 static void _DEMON_AXE_melee_effects(item_def* /*item*/, actor* attacker,
-                                     actor* /*defender*/, bool /*mondied*/,
+                                     actor* defender, bool /*mondied*/,
                                      int /*dam*/)
 {
+    const monster* mon = defender->as_monster();
+    if (mon && (mons_is_firewood(*mon) || mons_is_conjured(mon->type)))
+        return;
+
     if (one_chance_in(10))
     {
         if (monster* mons = attacker->as_monster())
@@ -1013,6 +1051,17 @@ static void _FIRESTARTER_melee_effects(item_def* /*weapon*/, actor* attacker,
     }
 }
 
+////////////////////////////////////////////////////
+static void _FORCE_LANCE_melee_effects(item_def* /*weapon*/, actor* attacker,
+                                       actor* defender, bool mondied, int dam)
+{
+    if (mondied || !dam || !one_chance_in(3)) return;
+    // max power around a !!! hit (ie ~2d11 on collide from a 34+ damage hit)
+    // no real justification for this, just vibes
+    const int collide_power = min(100, dam * 3);
+    defender->knockback(*attacker, 1, collide_power, "blow");
+}
+
 ///////////////////////////////////////////////////
 
 #if TAG_MAJOR_VERSION == 34
@@ -1362,9 +1411,6 @@ static void _ZHOR_world_reacts(item_def */*item*/)
 
 ////////////////////////////////////////////////////
 
-// XXX: Staff of Battle giving a boost to conjuration spells is hardcoded in
-// player_spec_conj().
-
 static void _BATTLE_unequip(item_def */*item*/, bool */*show_msgs*/)
 {
     end_battlesphere(find_battlesphere(&you), false);
@@ -1376,22 +1422,12 @@ static void _BATTLE_world_reacts(item_def */*item*/)
         && there_are_monsters_nearby(true, true, false)
         && stop_summoning_reason(MR_RES_POISON, M_FLIES).empty())
     {
-        cast_battlesphere(&you, calc_spell_power(SPELL_BATTLESPHERE, true),
-                          GOD_NO_GOD, false);
+        const int pow = div_rand_round(15 + you.skill(SK_CONJURATIONS, 15), 3);
+        cast_battlesphere(&you, pow, GOD_NO_GOD, false);
         did_god_conduct(DID_WIZARDLY_ITEM, 10);
     }
 }
 
-static void _BATTLE_melee_effects(item_def* /*weapon*/, actor* attacker,
-                                  actor* /*defender*/, bool /*mondied*/, int /*dam*/)
-{
-    if (attacker)
-    {
-        aim_battlesphere(attacker, SPELL_MAGIC_DART);
-        trigger_battlesphere(attacker);
-    }
-}
-
 ////////////////////////////////////////////////////
 
 static void _EMBRACE_unequip(item_def *item, bool *show_msgs)
@@ -1538,7 +1574,7 @@ static void _WUCAD_MU_equip(item_def */*item*/, bool *show_msgs,
 {
     if (you.has_mutation(MUT_HP_CASTING))
     {
-        _equip_mpr(show_msgs, "The staff is unable to connect with your "
+        _equip_mpr(show_msgs, "The crystal ball is unable to connect with your "
                               "magical essence.");
     }
 }
@@ -1594,6 +1630,8 @@ static void _POWER_GLOVES_unequip(item_def * /*item*/, bool *show_msgs)
         _equip_mpr(show_msgs, "The surge of magic dissipates.");
 }
 
+////////////////////////////////////////////////////
+
 static void _DREAMSHARD_NECKLACE_equip(item_def * /*item*/, bool *show_msgs,
                                       bool /*unmeld*/)
 {
@@ -1605,7 +1643,7 @@ static void _DREAMSHARD_NECKLACE_unequip(item_def * /* item */, bool * show_msgs
     _equip_mpr(show_msgs, "The world feels relentlessly logical and grey.");
 }
 
-//
+////////////////////////////////////////////////////
 
 static void _AUTUMN_KATANA_melee_effects(item_def* /*weapon*/, actor* attacker,
     actor* defender, bool /*mondied*/, int /*dam*/)
@@ -1665,6 +1703,7 @@ static void _AUTUMN_KATANA_melee_effects(item_def* /*weapon*/, actor* attacker,
 }
 
 ///////////////////////////////////////////////////
+
 static void _VITALITY_world_reacts(item_def */*item*/)
 {
     // once it starts regenerating you, you're doin evil
@@ -1674,3 +1713,113 @@ static void _VITALITY_world_reacts(item_def */*item*/)
         did_god_conduct(DID_EVIL, 1);
     }
 }
+
+///////////////////////////////////////////////////
+
+static void _reset_victory_stats(item_def *item)
+{
+    int &bonus_stats = item->props[VICTORY_STAT_KEY].get_int();
+    if (bonus_stats > 0)
+    {
+        bonus_stats = 0;
+        item->plus = get_unrand_entry(item->unrand_idx)->plus;
+        artefact_set_property(*item, ARTP_SLAYING, bonus_stats);
+        artefact_set_property(*item, ARTP_INTELLIGENCE, bonus_stats);
+        mprf(MSGCH_WARN, "%s stops glowing.", item->name(DESC_THE, false, true,
+                                                         false).c_str());
+
+        you.redraw_armour_class = true;
+        notify_stat_change();
+    }
+}
+
+static void _VICTORY_unequip(item_def *item, bool */*show_msgs*/)
+{
+    if (!player_equip_unrand(UNRAND_VICTORY, true))
+        _reset_victory_stats(item);
+}
+
+#define VICTORY_STAT_CAP 7
+
+static void _VICTORY_death_effects(item_def *item, monster* mons,
+                                   killer_type killer)
+{
+    // No bonus for killing friendlies, neutrals, summons, etc.
+    if (killer != KILL_YOU && killer != KILL_YOU_MISSILE
+        || !mons_gives_xp(*mons, you))
+    {
+        return;
+    }
+
+    const mon_threat_level_type threat = mons_threat_level(*mons);
+
+    // Increased chance of victory bonus from more dangerous mons.
+    // Using threat for this is kludgy, but easily visible to players.
+    if (threat == MTHRT_NASTY || (threat == MTHRT_TOUGH && x_chance_in_y(1, 4)))
+    {
+        int &bonus_stats = item->props[VICTORY_STAT_KEY].get_int();
+        if (bonus_stats < VICTORY_STAT_CAP)
+        {
+            bonus_stats++;
+            item->plus = bonus_stats;
+            artefact_set_property(*item, ARTP_SLAYING, bonus_stats);
+            artefact_set_property(*item, ARTP_INTELLIGENCE, bonus_stats);
+            mprf(MSGCH_GOD, GOD_OKAWARU, "%s glows%s.",
+                 item->name(DESC_THE, false, true, false).c_str(),
+                 bonus_stats == VICTORY_STAT_CAP ? " brightly" : "");
+
+            you.redraw_armour_class = true;
+            notify_stat_change();
+        }
+    }
+}
+
+static void _VICTORY_world_reacts(item_def *item)
+{
+    if (you.props.exists(VICTORY_CONDUCT_KEY))
+    {
+        _reset_victory_stats(item);
+        you.props.erase(VICTORY_CONDUCT_KEY);
+    }
+}
+
+static void _VICTORY_equip(item_def *item, bool */*show_msgs*/, bool /*unmeld*/)
+{
+    _VICTORY_world_reacts(item);
+}
+
+////////////////////////////////////////////////////
+
+static void _ASMODEUS_melee_effects(item_def* /*weapon*/, actor* attacker,
+                                    actor* defender, bool /*mondied*/,
+                                    int /*dam*/)
+{
+    if (!attacker->is_player() || you.allies_forbidden())
+        return;
+
+    const monster* mon = defender->as_monster();
+    if (mons_is_firewood(*mon)
+        || mons_is_conjured(mon->type)
+        || mon->is_summoned())
+    {
+        return;
+    }
+
+    if (one_chance_in(10))
+    {
+        const monster_type demon = random_choose_weighted(
+                                       3, MONS_BALRUG,
+                                       2, MONS_HELLION,
+                                       1, MONS_BRIMSTONE_FIEND);
+
+        mgen_data mg(demon, BEH_FRIENDLY, you.pos(), MHITYOU,
+                     MG_FORCE_BEH | MG_AUTOFOE);
+        mg.set_summoned(&you, 4, SPELL_FIRE_SUMMON);
+
+        if (create_monster(mg))
+        {
+            mpr("The sceptre summons one of its terrible servants.");
+            did_god_conduct(DID_EVIL, 3);
+        }
+    }
+}
