@@ -484,7 +484,7 @@ void ghost_demon::init_player_ghost()
                 case STAFF_POISON: brand = SPWPN_VENOM; break;
                 case STAFF_DEATH: brand = SPWPN_PAIN; break;
                 case STAFF_AIR: brand = SPWPN_ELECTROCUTION; break;
-                case STAFF_EARTH: brand = SPWPN_VORPAL; break;
+                case STAFF_EARTH: brand = SPWPN_HEAVY; break;
                 default: ;
                 }
             }
