@@ -71,6 +71,8 @@ InvEntry::InvEntry(const item_def &i)
     : MenuEntry("", MEL_ITEM), item(&i), _has_star(false)
 {
     indent_no_hotkeys = true;
+    // This gets the inventory coloring rules to apply by default:
+    tag = "inventory";
 
     // Data is an inherited void *. When using InvEntry in menus
     // use the const item in this class whenever possible
@@ -104,6 +106,11 @@ InvEntry::InvEntry(const item_def &i)
     quantity = i.quantity;
 }
 
+int InvEntry::highlight_colour(bool temp) const
+{
+    return menu_colour(get_text(), item_prefix(*item, temp), tag, false);
+}
+
 const string &InvEntry::get_basename() const
 {
     if (basename.empty())
@@ -307,10 +314,8 @@ InvMenu::InvMenu(int mflags)
         title_annotate(nullptr), _mode_special_drop(false)
 {
     menu_action = ACT_EXAMINE; // default
-#ifdef USE_TILE_LOCAL
-    if (Options.tile_menu_icons)
+    if (!Options.single_column_item_menus)
         set_flags(get_flags() | MF_USE_TWO_COLUMNS);
-#endif
 }
 
 bool InvMenu::mode_special_drop() const
@@ -414,27 +419,33 @@ void InvMenu::select_item_index(int idx, int qty)
 
 bool InvMenu::examine_index(int i)
 {
-    if (on_examine)
+    const bool do_actions = type == menu_type::describe;
+    // not entirely sure if the bounds check is necessary
+    auto ie = (i >= 0 && i < static_cast<int>(items.size()))
+                    ? dynamic_cast<InvEntry *>(items[i])
+                    : nullptr;
+
+    // superclass behavior: do nothing unless on_examine is defined, in which
+    // case call on_examine.
+    if (!ie || on_examine)
         return Menu::examine_index(i);
     else if (type == menu_type::pickup)
     {
-        auto ie = dynamic_cast<InvEntry *>(items[i]);
-        if (ie)
-        {
-            auto desc_tgt = const_cast<item_def*>(ie->item);
-            ASSERT(desc_tgt);
-            return describe_item(*desc_tgt);
-        }
+        // item is a floor item.
+        auto desc_tgt = const_cast<item_def*>(ie->item);
+        ASSERT(desc_tgt);
+        return describe_item(*desc_tgt, nullptr, do_actions);
     }
-    else if (i >= 0 && i < static_cast<int>(items.size()) && items[i]->hotkeys.size())
+    else if (ie->hotkeys.size())
     {
         // default behavior: examine inv item. You must override or use on_examine
         // if your items come from somewhere else, or this will cause crashes!
-        unsigned char select = items[i]->hotkeys[0];
+        unsigned char select = ie->hotkeys[0];
         const int invidx = letter_to_index(select);
         ASSERT(you.inv[invidx].defined());
-        return describe_item(you.inv[invidx]);
+        return describe_item(you.inv[invidx], nullptr, do_actions);
     }
+    // nothing to describe, ignore
     return true;
 }
 
@@ -494,7 +505,7 @@ string no_selectables_message(int item_selector)
             return "You aren't carrying any wearable armour.";
     }
     case OSEL_UNIDENT:
-        return "You don't have any unidentified items.";
+        return "You don't currently have any unidentified items.";
     case OSEL_ENCHANTABLE_ARMOUR:
         return "You aren't carrying any armour which can be enchanted further.";
     case OBJ_CORPSES:
@@ -523,6 +534,12 @@ string no_selectables_message(int item_selector)
 #endif
     case OSEL_WORN_ARMOUR:
         return "You aren't wearing any pieces of armour.";
+    case OSEL_WORN_JEWELLERY:
+        return "You aren't wearing any rings or amulets.";
+    case OSEL_WORN_EQUIPABLE:
+        return "You aren't wearing anything.";
+    case OSEL_EQUIPABLE:
+        return "You aren't carrying anything that can be equipped.";
     case OSEL_BRANDABLE_WEAPON:
         return "You aren't carrying any weapons that can be branded.";
     case OSEL_ENCHANTABLE_WEAPON:
@@ -615,7 +632,7 @@ bool get_tiles_for_item(const item_def &item, vector<tile_def>& tileset, bool sh
         }
     }
     if (item.base_type == OBJ_WEAPONS || item.base_type == OBJ_MISSILES
-        || item.base_type == OBJ_ARMOUR
+        || item.base_type == OBJ_ARMOUR || item.base_type == OBJ_STAVES
 #if TAG_MAJOR_VERSION == 34
         || item.base_type == OBJ_RODS
 #endif
@@ -625,12 +642,6 @@ bool get_tiles_for_item(const item_def &item, vector<tile_def>& tileset, bool sh
         if (brand)
             tileset.emplace_back(brand);
     }
-    else if (item.base_type == OBJ_CORPSES)
-    {
-        tileidx_t brand = tileidx_corpse_brand(item);
-        if (brand)
-            tileset.emplace_back(brand);
-    }
 
     return true;
 }
@@ -1086,7 +1097,8 @@ vector<SelItem> select_items(const vector<const item_def*> &items,
             new_flags |= MF_SELECT_QTY;
 
         new_flags |= MF_ALLOW_FORMATTING | MF_ARROWS_SELECT;
-        new_flags |= menu.get_flags() & MF_USE_TWO_COLUMNS;
+        if (!Options.single_column_item_menus)
+            new_flags |= menu.get_flags() & MF_USE_TWO_COLUMNS;
         menu.set_flags(new_flags);
         menu.show();
         selected = menu.get_selitems();
@@ -1128,8 +1140,7 @@ bool item_is_selected(const item_def &i, int selector)
         return item_is_wieldable(i);
 
     case OSEL_EVOKABLE:
-        // assumes valid link...would break with evoking from floor?
-        return item_is_evokable(i);//evoke_check(i.link, true);
+        return item_is_evokable(i);
 
     case OSEL_ENCHANTABLE_ARMOUR:
         return is_enchantable_armour(i, true);
@@ -1189,6 +1200,18 @@ bool item_is_selected(const item_def &i, int selector)
     case OSEL_QUIVER_ACTION_FORCE:
         return in_inventory(i) && quiver::slot_to_action(i.link, true)->is_valid();
 
+    case OSEL_WORN_JEWELLERY:
+        return item_is_equipped(i) && item_is_selected(i, OBJ_JEWELLERY);
+
+    case OSEL_WORN_EQUIPABLE:
+        if (!item_is_equipped(i))
+            return false;
+        // fallthrough
+    case OSEL_EQUIPABLE:
+        return item_is_selected(i, OBJ_ARMOUR)
+            || item_is_selected(i, OSEL_WIELD)
+            || item_is_selected(i, OBJ_JEWELLERY);
+
     default:
         return false;
     }
@@ -1324,12 +1347,7 @@ static string _drop_prompt(bool as_menu_title, bool menu_autopickup_mode)
         prompt_base = "Drop what?                               ";
     else
         prompt_base = "Drop what? ";
-    return prompt_base + slot_description()
-#ifdef TOUCH_UI
-                          + " (<Enter> or tap header to drop)";
-#else
-                          + " (_ for help)";
-#endif
+    return prompt_base + slot_description() + " (_ for help)";
 }
 
 static string _drop_menu_titlefn(const Menu *m, const string &)
@@ -1393,6 +1411,25 @@ item_def *digit_inscription_to_item(char digit, operation_types oper)
     return nullptr;
 }
 
+operation_types generalize_oper(operation_types oper)
+{
+    switch (oper)
+    {
+    case OPER_EQUIP:
+    case OPER_WIELD:
+    case OPER_WEAR:
+    case OPER_PUTON:
+        return OPER_EQUIP;
+    case OPER_UNEQUIP:
+    case OPER_REMOVE:
+    case OPER_TAKEOFF:
+        return OPER_UNEQUIP;
+    default:
+        return OPER_NONE;
+    }
+}
+
+
 static bool _has_warning_inscription(const item_def& item,
                              operation_types oper)
 {
@@ -1415,6 +1452,11 @@ static bool _has_warning_inscription(const item_def& item,
         }
     }
 
+    // if the inscription is wear/takeoff (etc), check equip/unequip
+    const auto gen = generalize_oper(oper);
+    if (gen != OPER_NONE && gen != oper)
+        return _has_warning_inscription(item, gen);
+
     return false;
 }
 
@@ -1472,7 +1514,7 @@ bool check_old_item_warning(const item_def& item,
                 return true;
 
             old_item = you.inv[equip];
-            if (!needs_handle_warning(old_item, OPER_TAKEOFF, penance))
+            if (!needs_handle_warning(old_item, OPER_REMOVE, penance))
                 return true;
 
             prompt += "Really remove ";
@@ -1509,8 +1551,9 @@ static string _operation_verb(operation_types oper)
     case OPER_ZAP:            return "zap";
     case OPER_FIRE:           return "fire";
     case OPER_EVOKE:          return "evoke";
-    case OPER_DESTROY:        return "destroy";
     case OPER_QUIVER:         return "quiver";
+    case OPER_EQUIP:          return "equip";
+    case OPER_UNEQUIP:        return "unequip";
     case OPER_ANY:
     default:
         return "choose";
@@ -1544,6 +1587,9 @@ bool needs_handle_warning(const item_def &item, operation_types oper,
     if (_has_warning_inscription(item, oper))
         return true;
 
+    // note: equip/unequip are not handled be the following code; they should
+    // be converted to their specific oper beforehand.
+
     // Curses first. Warn if something would take off (i.e. destroy) the cursed item.
     if (item.cursed()
         && (oper == OPER_WIELD && is_weapon(item)
@@ -1650,12 +1696,6 @@ bool check_warning_inscriptions(const item_def& item,
     if (item.defined()
         && needs_handle_warning(item, oper, penance))
     {
-        // When it's about destroying an item, don't even ask.
-        // If the player really wants to do that, they'll have
-        // to remove the inscription.
-        if (oper == OPER_DESTROY)
-            return false;
-
         // Common pattern for wield/wear/put:
         // - if the player isn't capable of equipping it, return true
         //   immediately. No point warning, since the op is impossible.
@@ -1978,82 +2018,6 @@ bool item_is_wieldable(const item_def &item)
     return is_weapon(item) && !you.has_mutation(MUT_NO_GRASPING);
 }
 
-/// Does the item only serve to produce summons or allies?
-static bool _item_ally_only(const item_def &item)
-{
-    if (item.base_type == OBJ_WANDS)
-        return item.sub_type == WAND_CHARMING;
-    else if (item.base_type == OBJ_MISCELLANY)
-    {
-        switch (item.sub_type)
-        {
-        case MISC_PHANTOM_MIRROR:
-        case MISC_HORN_OF_GERYON:
-        case MISC_BOX_OF_BEASTS:
-            return true;
-        default:
-            return false;
-        }
-    }
-    return false;
-}
-
-/**
- * Return whether an item can be evoked.
- *
- * @param item      The item to check
- * @param msg       Whether we need to print a message.
- */
-bool item_is_evokable(const item_def &item, bool msg)
-{
-    // XX unify with evoke_check?
-    const string error = item_is_melded(item)
-            ? "Your " + item.name(DESC_QUALNAME) + " is melded into your body."
-            : "That item can only be evoked when wielded.";
-
-    const bool no_evocables = you.get_mutation_level(MUT_NO_ARTIFICE);
-    const char* const no_evocable_error = "You cannot evoke magical items.";
-
-    if (no_evocables
-        && !(item.base_type == OBJ_MISCELLANY
-             && item.sub_type == MISC_ZIGGURAT)) // zigfigs are OK.
-    {
-        // the rest are forbidden under sac evocables.
-        if (msg)
-            mpr(no_evocable_error);
-        return false;
-    }
-
-    // TODO: check other summoning constraints here?
-    if (_item_ally_only(item) && you.allies_forbidden())
-    {
-        if (msg)
-            mpr("That item cannot be used by those who cannot gain allies!");
-        return false;
-    }
-
-    switch (item.base_type)
-    {
-    case OBJ_WANDS:
-        return true;
-
-    case OBJ_MISCELLANY:
-#if TAG_MAJOR_VERSION == 34
-        if (item.sub_type != MISC_BUGGY_LANTERN_OF_SHADOWS
-            && item.sub_type != MISC_BUGGY_EBONY_CASKET)
-#endif
-        {
-            return true;
-        }
-        // removed items fallthrough to failure
-
-    default:
-        if (msg)
-            mpr("That item cannot be evoked!");
-        return false;
-    }
-}
-
 /**
  * What xp-charged evocable items is the player currently devoting XP to, if
  * any?
