@@ -127,8 +127,9 @@ MIRET1(number, base_type, base_type)
  */
 MIRET1(number, number, number)
 /*** Does this monster have a ranged attack we know about?
- * This refers to ranged weapons. Ranged spells and abilities are not included
- * in this check.
+ * A monster is considered to have a ranged attack if it has any of the
+ * following: a reach attack, a throwable missile, a launcher weapon, an
+ * attack wand, or an attack spell with a range greater than 1.
  * @treturn boolean
  * @function has_ranged_attack
  */
@@ -270,7 +271,7 @@ static int moninf_get_defeat_wl(lua_State *ls)
     bool is_evoked = lua_isboolean(ls, 3) ? lua_toboolean(ls, 3) : false;
     int power = is_evoked ?
         (15 + you.skill(SK_EVOCATIONS, 7) / 2) * (wand_mp_cost() + 9) / 9 :
-        calc_spell_power(spell, true);
+        calc_spell_power(spell);
     spell_flags flags = get_spell_flags(spell);
     bool wl_check = testbits(flags, spflag::WL_check)
         && testbits(flags, spflag::dir_or_target)
