@@ -478,6 +478,10 @@ static bool _mutation_filter(string key, string /*body*/)
     return !strip_suffix(lowercase(key), " mutation");
 }
 
+static bool _passive_filter(string key, string /*body*/)
+{
+    return !strip_suffix(lowercase(key), " passive");
+}
 
 static void _recap_mon_keys(vector<string> &keys)
 {
@@ -846,26 +850,27 @@ static string _mons_desc_key(monster_type type)
 void LookupType::display_keys(vector<string> &key_list) const
 {
     DescMenu desc_menu(MF_SINGLESELECT | MF_ANYPRINTABLE | MF_ALLOW_FORMATTING
-                | MF_USE_TWO_COLUMNS | MF_ARROWS_SELECT,
+                | MF_USE_TWO_COLUMNS | MF_ARROWS_SELECT | MF_INIT_HOVER,
             toggleable_sort());
     desc_menu.set_tag("description");
 
     // XXX: ugh
     const bool doing_mons = type == "monster";
     vector<monster_info> monster_list(key_list.size());
+    int letter_i = 0;
     for (unsigned int i = 0, size = key_list.size(); i < size; i++)
     {
-        const char letter = index_to_letter(i % 52);
+        const char letter = index_to_letter(letter_i % 52);
         string &key = key_list[i];
         // XXX: double ugh
-        if (doing_mons)
-        {
-            desc_menu.add_entry(_monster_menu_gen(letter,
-                                                  key_to_menu_str(key),
-                                                  monster_list[i]));
-        }
-        else
-            desc_menu.add_entry(make_menu_entry(letter, key));
+        auto *entry = doing_mons
+            ? _monster_menu_gen(letter, key_to_menu_str(key), monster_list[i])
+            : make_menu_entry(letter, key);
+
+        if (!entry)
+            continue;
+        desc_menu.add_entry(entry);
+        letter_i++;
     }
 
     desc_menu.sort();
@@ -888,6 +893,10 @@ void LookupType::display_keys(vector<string> &key_list) const
         return true;
     };
 
+    // for some reason DescMenu is an InvMenu, so we need to do something to
+    // prevent examine crashes. Just alias it to regular selection.
+    desc_menu.on_examine = desc_menu.on_single_selection;
+
     while (true)
     {
         desc_menu.show();
@@ -1114,7 +1123,10 @@ static int _describe_item(const string &key, const string &suffix,
     {
         const int unrand_idx = extant_unrandart_by_exact_name(item_name);
         if (!unrand_idx)
-            die("Unable to get item %s by name", key.c_str());
+        {
+            ui::error(make_stringf("Unable to get item '%s' by name", key.c_str()));
+            return 0;
+        }
         _make_item_fake_unrandart(item, unrand_idx);
     }
     describe_item_popup(item);
@@ -1146,20 +1158,18 @@ static int _describe_god(const string &key, const string &/*suffix*/,
     return 0; // no exact matches for gods, so output doesn't matter
 }
 
-static string _branch_entry_runes(branch_type br)
+static string _branch_transit_runes(branch_type br)
 {
+    if (br != BRANCH_VAULTS && br != BRANCH_ZOT)
+        return "";
+
     string desc;
-    const int num_runes = runes_for_branch(br);
-
-    if (num_runes > 0)
-    {
-        desc = make_stringf("\n\nThis %s can only be entered while carrying "
-                            "at least %d rune%s of Zot.",
-                            br == BRANCH_ZIGGURAT ? "portal" : "branch",
-                            num_runes, num_runes > 1 ? "s" : "");
-    }
-
-    return desc;
+    const bool exit = br == BRANCH_VAULTS;
+    const int num_runes = br == BRANCH_ZOT ? 3 : 1;
+    return make_stringf("\n\nThis branch can only be %sed while carrying at "
+                        "least %d rune%s of Zot.",
+                        exit ? "exit" : "enter",
+                        num_runes, num_runes > 1 ? "s" : "");
 }
 
 static string _branch_depth(branch_type br)
@@ -1248,7 +1258,7 @@ static int _describe_branch(const string &key, const string &suffix,
         info += "\n\n" + noise_desc;
 
     info += _branch_location(branch)
-            + _branch_entry_runes(branch)
+            + _branch_transit_runes(branch)
             + _branch_depth(branch)
             + _branch_subbranches(branch)
             + "\n\n"
@@ -1258,6 +1268,16 @@ static int _describe_branch(const string &key, const string &suffix,
     return _describe_key(key, suffix, footer, info, &tile);
 }
 
+static int _describe_mutation(const string &key, const string &suffix,
+                              string /*footer*/)
+{
+    const string mutation_name = key.substr(0, key.size() - suffix.size());
+    const mutation_type mutation = mutation_from_name(mutation_name.c_str(),
+                                                      false);
+    describe_mutation(mutation);
+    return 0;
+}
+
 /// All types of ?/ queries the player can enter.
 static const vector<LookupType> lookup_types = {
     LookupType('M', "monster", _recap_mon_keys, _monster_filter,
@@ -1290,12 +1310,15 @@ static const vector<LookupType> lookup_types = {
     LookupType('L', "cloud", nullptr, nullptr,
                nullptr, _get_cloud_keys, _cloud_menu_gen,
                _describe_cloud, lookup_type::db_suffix),
+    LookupType('P', "passive", nullptr, _passive_filter,
+               nullptr, nullptr, _simple_menu_gen,
+               _describe_generic, lookup_type::db_suffix),
     LookupType('T', "status", nullptr, _status_filter,
                nullptr, nullptr, _simple_menu_gen,
                _describe_generic, lookup_type::db_suffix),
     LookupType('U', "mutation", nullptr, _mutation_filter,
                nullptr, nullptr, _mut_menu_gen,
-               _describe_generic, lookup_type::db_suffix),
+               _describe_mutation, lookup_type::db_suffix),
 };
 
 /**
