@@ -91,7 +91,7 @@ static const vector<pop_entry> population[] =
   {  4,  7, 1000, PEAK, MONS_IGUANA },
   {  4,  7,  350, PEAK, MONS_PHANTOM },
   {  4,  8,  500, PEAK, MONS_JELLY },
-  {  4,  8,  350, PEAK, MONS_NECROPHAGE },
+  {  4,  8,  350, PEAK, MONS_SLEEPCAP },
   {  4,  8,  350, PEAK, MONS_BLACK_BEAR },
 
   {  4, 10,  200, PEAK, MONS_GNOLL_BOUDA },
@@ -131,13 +131,13 @@ static const vector<pop_entry> population[] =
   { 11, 21,  825, SEMI, MONS_CYCLOPS },
   { 11, 21,  285, PEAK, MONS_BASILISK },
   { 11, 21,  192, PEAK, MONS_SKELETAL_WARRIOR },
-  { 11, 23,  540, SEMI, MONS_HYDRA },
   { 11, 23,  540, SEMI, MONS_BOULDER_BEETLE},
   { 12, 19,  475, SEMI, MONS_DEEP_ELF_FIRE_MAGE },
   { 12, 22,  440, FALL, MONS_SLIME_CREATURE },
   { 13, 20,  150, SEMI, MONS_MELIAI },
   { 13, 21,  192, PEAK, MONS_DEATH_KNIGHT },
   { 13, 21,   89, PEAK, MONS_RAKSHASA },
+  { 13, 23,  540, SEMI, MONS_HYDRA },
   { 13, 17,  925, FLAT, MONS_CENTAUR_WARRIOR },
   { 18, 23,  925, FALL, MONS_CENTAUR_WARRIOR },
   { 13, 19,  365, SEMI, MONS_TENGU_WARRIOR },
@@ -198,7 +198,6 @@ static const vector<pop_entry> population[] =
 },
 
 { // Orcish Mines
-  {  1,  4,  384, FLAT, MONS_NO_MONSTER },
   {  1,  4,  192, FLAT, MONS_ORC_WARRIOR },
   {  1,  4,   25, FLAT, MONS_ORC_PRIEST },
   {  1,  4,   25, FLAT, MONS_ORC_WIZARD },
@@ -276,7 +275,6 @@ static const vector<pop_entry> population[] =
   {  0,  5,  615, FLAT, MONS_KOMODO_DRAGON },
   {  0,  7,   50, PEAK, MONS_HELL_RAT },
   {  0,  7,  150, SEMI, MONS_POLAR_BEAR },
-  {  0, 10,  400, SEMI, MONS_HYDRA },
   {  1,  4,   16, PEAK, MONS_HORNET },
   {  1,  4,   50, PEAK, MONS_STEAM_DRAGON },
   {  1,  6,   25, PEAK, MONS_OKLOB_PLANT },
@@ -286,7 +284,9 @@ static const vector<pop_entry> population[] =
   {  1, 15,   70, SEMI, MONS_BOULDER_BEETLE },
   {  1, 15,   85, SEMI, MONS_TORPOR_SNAIL },
   {  2,  7,   75, SEMI, MONS_POLAR_BEAR },
+  {  2,  7,  200, SEMI, MONS_SKYSHARK },
   {  2,  9,   50, PEAK, MONS_LINDWURM },
+  {  2, 10,  400, SEMI, MONS_HYDRA },
   {  3, 11,  140, SEMI, MONS_DREAM_SHEEP },
   {  3, 15,   36, RISE, MONS_FIRE_CRAB },
   {  6,  9,  160, FLAT, MONS_CANE_TOAD },
@@ -353,6 +353,7 @@ static const vector<pop_entry> population[] =
 { // Shoals
   {  0,  3,   89, SEMI, MONS_CENTAUR_WARRIOR },
   {  0,  3,  355, SEMI, MONS_FAUN },
+  {  0,  3,  400, SEMI, MONS_SKYSHARK },
   {  0,  6,  300, SEMI, MONS_WATER_NYMPH },
   {  0,  6,  170, SEMI, MONS_MERFOLK_AVATAR },
   {  0,  6,  110, SEMI, MONS_CYCLOPS },
