@@ -1611,7 +1611,7 @@ static const mon_spellbook mspell_list[] =
     // (';') Walking Tomes.
     { MST_WALKING_TOME,
       {
-       { SPELL_CONJURE_LIVING_SPELLS, 60, MON_SPELL_MAGICAL },
+       { SPELL_CONJURE_LIVING_SPELLS, 60, MON_SPELL_WIZARD },
       }
     },
 
@@ -1785,7 +1785,8 @@ static const mon_spellbook mspell_list[] =
 
     { MST_BAI_SUZHEN_DRAGON,
       {
-        { SPELL_PRIMAL_WAVE, 60, MON_SPELL_NATURAL | MON_SPELL_BREATH },
+        { SPELL_PRIMAL_WAVE, 60, MON_SPELL_NATURAL | MON_SPELL_BREATH
+                                  | MON_SPELL_NOISY },
       }
     },
 
@@ -1793,7 +1794,7 @@ static const mon_spellbook mspell_list[] =
       {
        { SPELL_SERPENT_OF_HELL_GEH_BREATH, 35,
            MON_SPELL_NATURAL | MON_SPELL_BREATH | MON_SPELL_NOISY },
-       { SPELL_SUMMON_DRAGON, 35, MON_SPELL_MAGICAL },
+       { SPELL_SUMMON_DRAGON, 35, MON_SPELL_MAGICAL | MON_SPELL_NOISY },
       }
     },
 
@@ -1801,7 +1802,7 @@ static const mon_spellbook mspell_list[] =
       {
        { SPELL_SERPENT_OF_HELL_COC_BREATH, 35,
          MON_SPELL_NATURAL | MON_SPELL_BREATH | MON_SPELL_NOISY },
-       { SPELL_SUMMON_DRAGON, 35, MON_SPELL_MAGICAL },
+       { SPELL_SUMMON_DRAGON, 35, MON_SPELL_MAGICAL | MON_SPELL_NOISY },
       }
     },
 
@@ -1809,7 +1810,7 @@ static const mon_spellbook mspell_list[] =
       {
        { SPELL_SERPENT_OF_HELL_DIS_BREATH, 35,
          MON_SPELL_NATURAL | MON_SPELL_BREATH | MON_SPELL_NOISY },
-       { SPELL_SUMMON_DRAGON, 35, MON_SPELL_MAGICAL },
+       { SPELL_SUMMON_DRAGON, 35, MON_SPELL_MAGICAL | MON_SPELL_NOISY },
       }
     },
 
@@ -1817,7 +1818,7 @@ static const mon_spellbook mspell_list[] =
       {
        { SPELL_SERPENT_OF_HELL_TAR_BREATH, 35,
          MON_SPELL_NATURAL | MON_SPELL_BREATH | MON_SPELL_NOISY },
-       { SPELL_SUMMON_DRAGON, 35, MON_SPELL_MAGICAL },
+       { SPELL_SUMMON_DRAGON, 35, MON_SPELL_MAGICAL | MON_SPELL_NOISY },
       }
     },
 
