@@ -48,7 +48,7 @@ enum monster_type                      // env.mons[].type
 #endif
 #if TAG_MAJOR_VERSION > 34
     MONS_CRYSTAL_ECHIDNA,
-    MONS_PALENTONGA,
+    MONS_ARMATAUR,
 #endif
     MONS_JACKAL,
     MONS_HOUND,
@@ -352,6 +352,7 @@ enum monster_type                      // env.mons[].type
     MONS_FUNGUS,
     MONS_WANDERING_MUSHROOM,
 #if TAG_MAJOR_VERSION > 34
+    MONS_SLEEPCAP,
     MONS_DEATHCAP,
 #endif
     MONS_PLANT,
@@ -463,6 +464,8 @@ enum monster_type                      // env.mons[].type
     MONS_MERFOLK_AVATAR,
 #if TAG_MAJOR_VERSION > 34
     MONS_DROWNED_SOUL,
+    MONS_SICKLY_MERFOLK_SIREN,
+    MONS_MALARIOUS_MERFOLK_AVATAR,
 #endif
     MONS_MERFOLK_IMPALER,
     MONS_MERFOLK_AQUAMANCER,
@@ -597,6 +600,8 @@ enum monster_type                      // env.mons[].type
     MONS_SWAMP_WORM,
 #if TAG_MAJOR_VERSION == 34
     MONS_SHARK,
+#else
+    MONS_SKYSHARK,
 #endif
     MONS_KRAKEN,
     MONS_KRAKEN_TENTACLE,
@@ -621,6 +626,9 @@ enum monster_type                      // env.mons[].type
     MONS_UFETUBUS,
     MONS_IRON_IMP,
     MONS_SHADOW_IMP,
+#if TAG_MAJOR_VERSION > 34
+    MONS_CERULEAN_IMP,
+#endif
     MONS_RED_DEVIL,
 #if TAG_MAJOR_VERSION == 34
     MONS_ROTTING_DEVIL,
@@ -1123,7 +1131,7 @@ enum monster_type                      // env.mons[].type
     MONS_FOXFIRE,
     MONS_MAGGIE,
     MONS_NAMELESS,
-    MONS_PALENTONGA,
+    MONS_ARMATAUR,
     MONS_BLOATED_HUSK,
     MONS_BUNYIP,
     MONS_GOLIATH_FROG,
@@ -1177,6 +1185,11 @@ enum monster_type                      // env.mons[].type
     MONS_METEORAN,
     MONS_GRUNN,
     MONS_STARFLOWER,
+    MONS_SICKLY_MERFOLK_SIREN,
+    MONS_MALARIOUS_MERFOLK_AVATAR,
+    MONS_SKYSHARK,
+    MONS_SLEEPCAP,
+    MONS_CERULEAN_IMP,
 #endif
 
     NUM_MONSTERS,               // used for polymorph
