@@ -50,6 +50,7 @@ static const mutation_def mut_data[] =
    "Your muscles are very strong. (Str +8, Int/Dex -2)", ""},
   {"", "", ""},
   {"", "", ""},
+  TILEG_MUT_STRONG,
 },
 
 { MUT_CLEVER, 7, 2, mutflag::good, false,
@@ -59,6 +60,7 @@ static const mutation_def mut_data[] =
    "Your mind is very acute. (Int +8, Str/Dex -2)", ""},
   {"", "", ""},
   {"", "", ""},
+  TILEG_MUT_CLEVER,
 },
 
 { MUT_AGILE, 7, 2, mutflag::good, false,
@@ -68,6 +70,7 @@ static const mutation_def mut_data[] =
    "You are very agile. (Dex +8, Str/Int -2)", ""},
   {"", "", ""},
   {"", "", ""},
+  TILEG_MUT_AGILE,
 },
 
 { MUT_POISON_RESISTANCE, 4, 1, mutflag::good, true,
@@ -76,6 +79,7 @@ static const mutation_def mut_data[] =
   {"Your system is resistant to poisons. (rPois)", "", ""},
   {"You feel resistant to poisons.", "",  ""},
   {"You feel less resistant to poisons.", "", ""},
+  TILEG_MUT_POISON_RESISTANCE,
 },
 #if TAG_MAJOR_VERSION == 34
 
@@ -113,6 +117,7 @@ static const mutation_def mut_data[] =
   {"You no longer feel heat resistant.",
    "You feel less heat resistant.",
    "You feel less heat resistant."},
+  TILEG_MUT_HEAT_RESISTANCE,
 },
 
 { MUT_COLD_RESISTANCE, 4, 3, mutflag::good, true,
@@ -129,9 +134,10 @@ static const mutation_def mut_data[] =
   {"You no longer feel cold resistant.",
    "You feel less cold resistant.",
    "You feel less cold resistant."},
+  TILEG_MUT_COLD_RESISTANCE,
 },
 
-{ MUT_HEAT_VULNERABILITY, 0, 3,
+{ MUT_HEAT_VULNERABILITY, 3, 3,
   mutflag::bad | mutflag::qazlal, true,
   "heat vulnerability",
 
@@ -146,9 +152,10 @@ static const mutation_def mut_data[] =
   {"You no longer feel vulnerable to heat.",
    "You feel less vulnerable to heat.",
    "You feel less vulnerable to heat."},
+  TILEG_MUT_HEAT_VULNERABILITY,
 },
 
-{ MUT_COLD_VULNERABILITY, 0, 3,
+{ MUT_COLD_VULNERABILITY, 3, 3,
   mutflag::bad | mutflag::qazlal, true,
 
   "cold vulnerability",
@@ -164,6 +171,7 @@ static const mutation_def mut_data[] =
   {"You no longer feel vulnerable to cold.",
    "You feel less vulnerable to cold.",
    "You feel less vulnerable to cold."},
+  TILEG_MUT_COLD_VULNERABILITY,
 },
 
 { MUT_DEMONIC_GUARDIAN, 0, 3, mutflag::good, false,
@@ -188,6 +196,7 @@ static const mutation_def mut_data[] =
   {"You are resistant to electric shocks. (rElec)", "", ""},
   {"You feel insulated.", "", ""},
   {"You feel conductive.", "", ""},
+  TILEG_MUT_SHOCK_RESISTANCE,
 },
 
 { MUT_SHOCK_VULNERABILITY, 0, 1, mutflag::bad | mutflag::qazlal, true,
@@ -196,6 +205,7 @@ static const mutation_def mut_data[] =
   {"You are vulnerable to electric shocks.", "", ""},
   {"You feel vulnerable to electricity.", "", ""},
   {"You feel less vulnerable to electricity.", "", ""},
+  TILEG_MUT_SHOCK_VULNERABILITY,
 },
 
 { MUT_REGENERATION, 2, 3, mutflag::good, false,
@@ -258,31 +268,34 @@ static const mutation_def mut_data[] =
 },
 #endif
 
-{ MUT_WEAK, 8, 2, mutflag::bad | mutflag::xom, false,
+{ MUT_WEAK, 8, 2, mutflag::bad, false,
   "weak",
 
   {"You are weak. (Str -2)",
    "You are very weak. (Str -4)", ""},
   {"", "", ""},
   {"", "", ""},
+  TILEG_MUT_WEAK,
 },
 
-{ MUT_DOPEY, 8, 2, mutflag::bad | mutflag::xom, false,
+{ MUT_DOPEY, 8, 2, mutflag::bad, false,
   "dopey",
 
   {"You are dopey. (Int -2)",
    "You are very dopey. (Int -4)", ""},
   {"", "", ""},
   {"", "", ""},
+  TILEG_MUT_DOPEY,
 },
 
-{ MUT_CLUMSY, 8, 2, mutflag::bad | mutflag::xom, false,
+{ MUT_CLUMSY, 8, 2, mutflag::bad, false,
   "clumsy",
 
   {"You are clumsy. (Dex -2)",
    "You are very clumsy. (Dex -4)", ""},
   {"", "", ""},
   {"", "", ""},
+  TILEG_MUT_CLUMSY,
 },
 #if TAG_MAJOR_VERSION == 34
 
@@ -369,9 +382,7 @@ static const mutation_def mut_data[] =
   TILEG_MUT_ACUTE_VISION,
 },
 
-{ MUT_DEFORMED, 8, 1,
-  mutflag::bad | mutflag::xom,
-  true,
+{ MUT_DEFORMED, 8, 1, mutflag::bad, true,
   "deformed body",
 
   {"Armour fits poorly on your strangely shaped body.", "", ""},
@@ -470,7 +481,7 @@ static const mutation_def mut_data[] =
 },
 #endif
 
-{ MUT_SCREAM, 6, 2, mutflag::bad | mutflag::xom, false,
+{ MUT_SCREAM, 6, 2, mutflag::bad, false,
   "screaming",
 
   {"You occasionally shout uncontrollably at your foes.",
@@ -522,7 +533,7 @@ static const mutation_def mut_data[] =
    ""},
 },
 
-{ MUT_DETERIORATION, 10, 2, mutflag::bad | mutflag::xom, false,
+{ MUT_DETERIORATION, 10, 2, mutflag::bad, false,
   "deterioration",
 
   {"Your body sometimes deteriorates upon taking damage.",
@@ -570,6 +581,7 @@ static const mutation_def mut_data[] =
   {"You feel genetically unstable.",
    "You feel genetically unstable.",
    "You feel genetically unstable."},
+  TILEG_MUT_MUTATION_RESISTANCE,
 },
 
 { MUT_EVOLUTION, 4, 2, mutflag::good, false,
@@ -604,8 +616,7 @@ static const mutation_def mut_data[] =
    ""},
 },
 
-{ MUT_FRAIL, 10, 3,
-  mutflag::bad | mutflag::xom, false,
+{ MUT_FRAIL, 10, 3, mutflag::bad, false,
   "frail",
 
   {"You are frail. (-10% HP)",
@@ -672,6 +683,7 @@ static const mutation_def mut_data[] =
    "You feel more resistant to negative energy."},
 
   {"", "", ""},
+  TILEG_MUT_NEGATIVE_ENERGY_RESISTANCE,
 },
 #if TAG_MAJOR_VERSION == 34
 
@@ -939,9 +951,10 @@ static const mutation_def mut_data[] =
 
   {"", "", ""},
 },
+#if TAG_MAJOR_VERSION == 34
 
-// Palentonga only
-{ MUT_ROLL, 8, 3, mutflag::good, true,
+// Armataur only
+{ MUT_ROLL, 0, 3, mutflag::good, true,
   "roll",
 
   {"You can roll at nearby foes to attack.",
@@ -957,6 +970,15 @@ static const mutation_def mut_data[] =
    "You can no longer roll as far."},
 },
 
+{ MUT_CURL, 0, 1, mutflag::good, true,
+  "reflexive curl",
+
+  {"You curl defensively after being hit. (AC +7*)", "", ""},
+  {"You now curl defensively after being hit.", "", ""},
+  {"", "", ""},
+},
+#endif
+
 { MUT_ARMOURED_TAIL, 0, 1, mutflag::good, true,
   "armoured tail",
 
@@ -965,15 +987,14 @@ static const mutation_def mut_data[] =
   {"", "", ""},
 },
 
-{ MUT_CURL, 0, 1, mutflag::good, true,
-  "reflexive curl",
+{ MUT_ROLLPAGE, 0, 1, mutflag::good, false,
+  "roll",
 
-  {"You curl defensively after being hit. (AC +7*)", "", ""},
-  {"You now curl defensively after being hit.", "", ""},
+  {"You roll when moving toward enemies. (Rampage)", "", ""},
+  {"", "", ""},
   {"", "", ""},
 },
 
-
 { MUT_SHAGGY_FUR, 2, 3, mutflag::good, true,
   "shaggy fur",
 
@@ -1848,7 +1869,7 @@ static const mutation_def mut_data[] =
 },
 #endif
 
-{ MUT_DRINK_SAFETY, 7, 2, mutflag::bad | mutflag::xom, false,
+{ MUT_DRINK_SAFETY, 7, 2, mutflag::bad, false,
   "inability to drink after injury",
 
   {"You occasionally lose the ability to drink potions when taking damage.",
@@ -1862,7 +1883,7 @@ static const mutation_def mut_data[] =
    ""},
 },
 
-{ MUT_READ_SAFETY, 7, 2, mutflag::bad | mutflag::xom, false,
+{ MUT_READ_SAFETY, 7, 2, mutflag::bad, false,
   "inability to read after injury",
 
   {"You occasionally lose the ability to read scrolls when taking damage.",
@@ -2178,7 +2199,7 @@ static const mutation_def mut_data[] =
   {"You grow two extra arms.", "", ""},
 },
 
-{ MUT_NO_DRINK, 0, 1, mutflag::good, false,
+{ MUT_NO_DRINK, 0, 1, mutflag::bad, false,
   "no potions",
 
   {"You do not drink.", "", ""},
@@ -2186,6 +2207,14 @@ static const mutation_def mut_data[] =
   {"You gain the ability to drink.", "", ""},
 },
 
+{ MUT_FAITH, 0, 1, mutflag::bad, false,
+  "faith",
+
+  {"You have a special connection with the divine. (Faith)", "", ""},
+  {"You feel connected to something greater than you.", "", ""},
+  {"You feel rebellious.", "", ""},
+},
+
 { MUT_REFLEXIVE_HEADBUTT, 0, 1, mutflag::good, true,
   "retaliatory headbutt",
 
@@ -2356,6 +2385,18 @@ static const mutation_def mut_data[] =
   {"You feel like a homebody.", "", ""},
 },
 
+{ MUT_LONG_TONGUE, 0, 1, mutflag::good, false, "long tongue",
+  {"Your long tongue fully drains potion bottles. (2x potion effects)", "", ""},
+  {"Your tongue grows exceptionally long.", "", ""},
+  {"Your tongue shrinks into a sad, ordinary nub.", "", ""},
+},
+
+{ MUT_AWKWARD_TONGUE, 0, 1, mutflag::bad, false, "awkward tongue",
+  {"Your tongue gives you trouble enunciating. (1.5x scroll delay)", "", ""},
+  {"Your tongue begins to flop around amusingly.", "", ""},
+  {"Your tongue regains its customary placidity.", "", ""},
+},
+
 };
 
 static const mutation_category_def category_mut_data[] =
