@@ -69,11 +69,8 @@ static void _give_wanderer_weapon(skill_type wpn_skill, bool good_item)
         sub_type = WPN_SPEAR;
         break;
 
-    // remaining types can't have basetype upgraded, so offer vorpal instead
     case SK_STAVES:
         sub_type = WPN_QUARTERSTAFF;
-        if (upgrade_base)
-            ego = SPWPN_VORPAL;
         break;
 
     case SK_RANGED_WEAPONS:
@@ -82,19 +79,22 @@ static void _give_wanderer_weapon(skill_type wpn_skill, bool good_item)
 
     default:
         sub_type = WPN_DAGGER;
-        if (upgrade_base)
-            ego = SPWPN_VORPAL;
         break;
     }
 
     if (upgrade_base)
     {
+        const weapon_type old_type = sub_type;
         sub_type = starting_weapon_upgrade(sub_type, you.char_class,
                                             you.species);
+        // If we can't upgrade the type, give it a little plus.
+        if (sub_type == old_type)
+            plus = 2;
     }
     else if (good_item)
         plus = 2;
 
+
     newgame_make_item(OBJ_WEAPONS, sub_type, 1, plus, ego);
 }
 
@@ -423,9 +423,10 @@ static void _good_potion_or_scroll()
 {
     // vector of weighted {object_class_type, subtype} pairs
     // xxx: could we use is_useless_item here? (not without dummy items...?)
+    const int ally_scr_type = item_for_set(ITEM_SET_ALLY_SCROLLS);
     const vector<pair<pair<object_class_type, int>, int>> options = {
         { { OBJ_SCROLLS, SCR_FEAR }, 4 },
-        { { OBJ_SCROLLS, SCR_SUMMONING }, 1 },
+        { { OBJ_SCROLLS, ally_scr_type }, 1 },
         { { OBJ_SCROLLS, SCR_BLINKING },
             you.stasis() ? 0 : 4 },
         { { OBJ_POTIONS, POT_HEAL_WOUNDS },
@@ -460,11 +461,10 @@ static void _decent_potion_or_scroll()
 {
     // vector of weighted {object_class_type, subtype} pairs
     // xxx: could we use is_useless_item here? (not without dummy items...?)
-    const int conceal_scr_type = item_for_set(ITEM_SET_CONCEAL_SCROLLS);
     const vector<pair<pair<object_class_type, int>, int>> options = {
         { { OBJ_SCROLLS, SCR_TELEPORTATION },
             you.stasis() ? 0 : 6 },
-        { { OBJ_SCROLLS, conceal_scr_type }, 6 },
+        { { OBJ_SCROLLS, SCR_FOG }, 6 },
         { { OBJ_SCROLLS, SCR_VULNERABILITY }, 2 },
         { { OBJ_SCROLLS, SCR_SILENCE }, 2 },
         { { OBJ_POTIONS, POT_CURING },
@@ -488,10 +488,12 @@ static void _wanderer_random_evokable()
 {
     if (one_chance_in(3))
     {
-        int selected_evoker =
+        const auto area_evoker_type =
+            (misc_item_type)item_for_set(ITEM_SET_AREA_MISCELLANY);
+        misc_item_type selected_evoker =
               random_choose(MISC_BOX_OF_BEASTS, MISC_PHIAL_OF_FLOODS,
-                            MISC_PHANTOM_MIRROR, MISC_CONDENSER_VANE,
-                            MISC_TIN_OF_TREMORSTONES, MISC_LIGHTNING_ROD);
+                            MISC_PHANTOM_MIRROR, area_evoker_type,
+                            MISC_LIGHTNING_ROD);
 
         newgame_make_item(OBJ_MISCELLANY, selected_evoker, 1);
     }
