@@ -31,4 +31,7 @@ enum object_selector
     OSEL_UNCURSED_WORN_RINGS     = -19,
     OSEL_QUIVER_ACTION           = -20,
     OSEL_QUIVER_ACTION_FORCE     = -21,
+    OSEL_EQUIPABLE               = -22, // armour, jewellery, weapons
+    OSEL_WORN_JEWELLERY          = -23,
+    OSEL_WORN_EQUIPABLE          = -24,
 };
