@@ -319,7 +319,7 @@ static void _equip_artefact_effect(item_def &item, bool *show_msgs, bool unmeld,
     if (proprt[ARTP_CONTAM] && msg && !unmeld)
         mpr("You feel a build-up of mutagenic energy.");
 
-    if (proprt[ARTP_RAMPAGING] && msg && !unmeld)
+    if (proprt[ARTP_RAMPAGING] && msg && !unmeld && !you.has_mutation(MUT_ROLLPAGE))
         mpr("You feel ready to rampage towards enemies.");
 
     if (proprt[ARTP_ARCHMAGI] && msg && !unmeld)
@@ -456,6 +456,8 @@ static void _equip_weapon_effect(item_def& item, bool showMsgs, bool unmeld)
     {
     case OBJ_STAVES:
     {
+        if (artefact)
+            _equip_artefact_effect(item, &showMsgs, unmeld, EQ_STAFF);
         break;
     }
 
@@ -691,10 +693,7 @@ static void _unequip_weapon_effect(item_def& real_item, bool showMsgs,
                 {
                     monster *spectral_weapon = find_spectral_weapon(&you);
                     if (spectral_weapon)
-                    {
-                        mpr("Your spectral weapon disappears.");
-                        end_spectral_weapon(spectral_weapon, false, true);
-                    }
+                        end_spectral_weapon(spectral_weapon, false, false);
                 }
                 break;
 
@@ -831,7 +830,8 @@ static void _equip_armour_effect(item_def& arm, bool unmeld,
             break;
 
         case SPARM_RAMPAGING:
-            mpr("You feel ready to rampage towards enemies.");
+            if (!you.has_mutation(MUT_ROLLPAGE))
+                mpr("You feel ready to rampage towards enemies.");
             break;
 
         case SPARM_INFUSION:
@@ -1153,6 +1153,11 @@ static void _equip_jewellery_effect(item_def &item, bool unmeld,
             mpr("You feel a surge of self-confidence.");
             break;
         }
+        if (you.has_mutation(MUT_FAITH))
+        {
+            mpr("You already have all the faith you need.");
+            break;
+        }
 
         const string ignore_reason = ignore_faith_reason();
         if (!ignore_reason.empty())
