@@ -11,6 +11,7 @@
 #include <cstring>
 #include <unordered_map>
 
+#include "art-enum.h"
 #include "cloud.h"
 #include "god-conduct.h"
 #include "god-passive.h"
@@ -33,13 +34,20 @@
 #include "transform.h"
 #include "xom.h"
 
+static int _scale_effect(int base, bool is_potion)
+{
+    if (!is_potion || !you.has_mutation(MUT_LONG_TONGUE))
+        return base;
+    return base * 2;
+}
+
 int _xom_factor(bool was_known);
 
 PotionEffect::PotionEffect(const potion_type pot)
     : potion_name(potion_type_name(pot)), kind(pot)
 { }
 
-bool PotionEffect::can_quaff(string */*reason*/) const
+bool PotionEffect::can_quaff(string */*reason*/, bool /*temp*/) const
 {
     return true;
 }
@@ -75,11 +83,16 @@ public:
         static PotionCuring inst; return inst;
     }
 
-    bool can_quaff(string *reason = nullptr) const override
+    bool can_quaff(string *reason = nullptr, bool temp = true) const override
     {
-        // cure status effects
+        // no species that can quaff at all have permanent restrictions for
+        // curing
+        if (!temp)
+            return true;
+
+        // cure status effects, allowed even in death's door
         if (you.duration[DUR_CONF]
-            || you.duration[DUR_POISONING])
+                    || you.duration[DUR_POISONING])
         {
             return true;
         }
@@ -87,10 +100,10 @@ public:
         if (you.duration[DUR_DEATHS_DOOR])
         {
             if (reason)
-                *reason = "You can't heal while in death's door.";
+                *reason = "You cannot heal while in death's door.";
             return false;
         }
-        if (!you.can_potion_heal())
+        if (!you.can_potion_heal(true) || temp && you.hp == you.hp_max)
         {
             if (reason)
                 *reason = "You have no ailments to cure.";
@@ -105,9 +118,14 @@ public:
 
         if ((you.can_potion_heal() || !is_potion) && !ddoor)
         {
-            int amount = 5 + random2(7);
+            const int base = 5 + random2(7);
+            int amount = base;
             if (is_potion)
+            {
                 amount = you.scale_potion_healing(amount);
+                if (you.has_mutation(MUT_LONG_TONGUE))
+                    amount += base;
+            }
             inc_hp(amount);
         }
 
@@ -144,21 +162,28 @@ public:
         static PotionHealWounds inst; return inst;
     }
 
-    bool can_quaff(string *reason = nullptr) const override
+    bool can_quaff(string *reason = nullptr, bool temp = true) const override
     {
-        if (!you.can_potion_heal())
+        // note: equip, i.e. vines, leads to temp uselessness. Should this act
+        // as permauselessness?
+        if (!you.can_potion_heal(temp))
         {
             if (reason)
-                *reason = "You cannot be healed by potions.";
+            {
+                if (!temp || !you.can_potion_heal(false))
+                    *reason = "You cannot be healed by potions.";
+                else
+                    *reason = "You cannot currently be healed by potions.";
+            }
             return false;
         }
-        if (you.duration[DUR_DEATHS_DOOR])
+        if (temp && you.duration[DUR_DEATHS_DOOR])
         {
             if (reason)
                 *reason = "You cannot heal while in death's door.";
             return false;
         }
-        if (you.hp == you.hp_max)
+        if (temp && you.hp == you.hp_max)
         {
             if (reason)
                 *reason = "Your health is already full.";
@@ -180,9 +205,14 @@ public:
             return false;
         }
 
-        int amount = 10 + random2avg(28, 3);
+        const int base = 10 + random2avg(28, 3);
+        int amount = base;
         if (is_potion)
+        {
             amount = you.scale_potion_healing(amount);
+            if (you.has_mutation(MUT_LONG_TONGUE))
+                amount += base;
+        }
         inc_hp(amount);
         if (is_potion)
             print_potion_heal_message();
@@ -202,7 +232,7 @@ public:
         static PotionHaste inst; return inst;
     }
 
-    bool can_quaff(string *reason = nullptr) const override
+    bool can_quaff(string *reason = nullptr, bool /* temp */ = true) const override
     {
         if (you.stasis())
         {
@@ -213,9 +243,9 @@ public:
         return true;
     }
 
-    bool effect(bool=true, int pow = 40, bool=true) const override
+    bool effect(bool=true, int pow = 40, bool is_potion=true) const override
     {
-        return haste_player(40 + random2(pow));
+        return haste_player(_scale_effect(40 + random2(pow), is_potion));
     }
 
     bool quaff(bool was_known) const override
@@ -240,13 +270,14 @@ public:
         static PotionMight inst; return inst;
     }
 
-    bool effect(bool=true, int pow = 40, bool=true) const override
+    bool effect(bool=true, int pow = 40, bool is_potion=true) const override
     {
         const bool were_mighty = you.duration[DUR_MIGHT] > 0;
 
         mprf(MSGCH_DURATION, "You feel %s all of a sudden.",
              were_mighty ? "mightier" : "very mighty");
-        you.increase_duration(DUR_MIGHT, 35 + random2(pow), 80);
+        const int add = _scale_effect(35 + random2(pow), is_potion);
+        you.increase_duration(DUR_MIGHT, add, 80);
         return true;
     }
 };
@@ -262,13 +293,33 @@ public:
         static PotionBrilliance inst; return inst;
     }
 
-    bool effect(bool=true, int pow = 40, bool=true) const override
+    bool can_quaff(string *reason = nullptr, bool temp = true) const override
+    {
+        if (you_worship(GOD_TROG))
+        {
+            // technically can work under Trog, but it does nothing; so give
+            // an informative message instead.
+            if (reason)
+                *reason = "Trog doesn't allow you to cast spells!";
+            return false;
+        }
+        if (temp && player_equip_unrand(UNRAND_FOLLY))
+        {
+            if (reason)
+                *reason = "Your robe already provides the effects of brilliance.";
+            return false;
+        }
+        return true;
+    }
+
+    bool effect(bool=true, int pow = 40, bool is_potion=true) const override
     {
         const bool were_brilliant = you.duration[DUR_BRILLIANCE] > 0;
 
         mprf(MSGCH_DURATION, "You feel %sclever all of a sudden.",
              were_brilliant ? "more " : "");
-        you.increase_duration(DUR_BRILLIANCE, 35 + random2(pow), 80);
+        const int add = _scale_effect(35 + random2(pow), is_potion);
+        you.increase_duration(DUR_BRILLIANCE, add, 80);
         return true;
     }
 };
@@ -284,14 +335,19 @@ public:
         static PotionAttraction inst; return inst;
     }
 
-    bool effect(bool=true, int pow = 40, bool=true) const override
+    // note on uselessness: this potion works by status, so should still be
+    // allowed with no monsters in LOS. Because it is marked as dangerous,
+    // it always prompts. XX maybe add info to the prompt?
+
+    bool effect(bool=true, int pow = 40, bool is_potion=true) const override
     {
         const bool was_attractive = you.duration[DUR_ATTRACTIVE] > 0;
 
         mprf(MSGCH_DURATION, "You feel %sattractive to monsters.",
              was_attractive ? "more " : "");
 
-        you.increase_duration(DUR_ATTRACTIVE, 20 + random2(pow)/2);
+        const int add = _scale_effect(20 + random2(pow)/2, is_potion);
+        you.increase_duration(DUR_ATTRACTIVE, add);
         return true;
     }
 };
@@ -308,20 +364,26 @@ public:
         static PotionFlight inst; return inst;
     }
 
-    bool can_quaff(string *reason = nullptr) const override
+    bool can_quaff(string *reason = nullptr, bool temp = true) const override
     {
-        if (!flight_allowed(true, reason))
+        if (you.permanent_flight(false))
         {
             if (reason)
-                *reason = "You cannot fly right now.";
+                *reason = "You can already fly.";
             return false;
         }
+        // temp flight inhibition; example: form prevents flight
+        if (temp && !flight_allowed(true, reason))
+            return false;
+
+        // other tempflight cases: allowed and adds time
+
         return true;
     }
 
-    bool effect(bool=true, int pow = 40, bool=true) const override
+    bool effect(bool=true, int pow = 40, bool is_potion=true) const override
     {
-        fly_player(pow);
+        fly_player(_scale_effect(pow, is_potion));
         return you.airborne();
     }
 };
@@ -337,31 +399,24 @@ public:
         static PotionCancellation inst; return inst;
     }
 
-    bool can_quaff(string *reason = nullptr) const override
+    bool can_quaff(string *reason = nullptr, bool temp=true) const override
     {
-        if (!player_is_cancellable())
+        if (temp && !player_is_cancellable())
         {
             if (reason)
-                *reason = "Drinking this now will have no effect.";
+                *reason = "There is nothing to cancel.";
             return false;
         }
 
         return true;
     }
 
-    bool quaff(bool was_known) const override {
-        if (was_known && !check_known_quaff())
-            return false;
-
-        return effect(was_known);
-    }
-
-    bool effect(bool=true, int=40, bool=true) const override
+    bool effect(bool=true, int=40, bool is_potion=true) const override
     {
         debuff_player();
         mpr("You feel magically purged.");
         const int old_contam_level = get_contamination_level();
-        contaminate_player(-1 * (1000 + random2(4000)));
+        contaminate_player(-1 * _scale_effect(1000 + random2(4000), is_potion));
         if (old_contam_level && old_contam_level == get_contamination_level())
             mpr("You feel slightly less contaminated with magical energies.");
         return true;
@@ -391,6 +446,8 @@ public:
             return true;
         }
 
+        // should be unreachable: nothing blocks intentional confusion. (If
+        // this ever changes, consider adding a `can_quaff`)
         mpr("You feel briefly invigorated.");
         return false;
     }
@@ -407,7 +464,7 @@ public:
         static PotionInvisibility inst; return inst;
     }
 
-    bool effect(bool=true, int pow = 40, bool=true) const override
+    bool effect(bool=true, int pow = 40, bool is_potion=true) const override
     {
         if (you.backlit())
         {
@@ -441,26 +498,36 @@ public:
 
         // Now multiple invisiblity casts aren't as good. -- bwr
         if (!you.duration[DUR_INVIS])
-            you.set_duration(DUR_INVIS, 15 + random2(pow), 100);
+            you.set_duration(DUR_INVIS, _scale_effect(15 + random2(pow), is_potion), 100);
         else
-            you.increase_duration(DUR_INVIS, random2(pow), 100);
+            you.increase_duration(DUR_INVIS, _scale_effect(random2(pow), is_potion), 100);
         return true;
     }
 
-    bool can_quaff(string *reason = nullptr) const override
+    bool check_known_quaff() const override
     {
-        return invis_allowed(true, reason);
-    }
-
-    bool quaff(bool was_known) const override
-    {
-        // Let invis_allowed print the messages and possibly do a prompt.
-        if (was_known && !invis_allowed())
+        string reason;
+        // invis can be drunk while temp useless, if the player accepts at the
+        // prompt
+        if (!can_quaff(&reason, false))
+        {
+            mpr(reason);
             return false;
-
-        effect();
+        }
+        if (!can_quaff(&reason, true)
+             && !yesno((reason + " Use anyway?").c_str(), false, 'n'))
+        {
+            canned_msg(MSG_OK);
+            return false;
+        }
         return true;
     }
+
+    bool can_quaff(string *reason = nullptr, bool temp = true) const override
+    {
+        // quaffing invis
+        return invis_allowed(true, reason, temp);
+    }
 };
 
 class PotionExperience : public PotionEffect
@@ -474,7 +541,7 @@ public:
         static PotionExperience inst; return inst;
     }
 
-    bool effect(bool=true, int pow = 40, bool=true) const override
+    bool effect(bool=true, int pow = 40, bool is_potion=true) const override
     {
         if (player_under_penance(GOD_HEPLIAKLQANA))
         {
@@ -487,7 +554,7 @@ public:
 
         if (you.experience_level < you.get_max_xl())
         {
-            const int levels = min(you.get_max_xl(), pow / 40);
+            const int levels = min(you.get_max_xl(), _scale_effect(pow / 40, is_potion));
             mpr("You feel more experienced!");
             // Defer calling level_change() until later in drink() to prevent
             // SIGHUP abuse.
@@ -497,7 +564,7 @@ public:
             mpr("A flood of memories washes over you.");
 
         // these are included in default force_more_message
-        const int exp = 7500 * you.experience_level * pow / 40;
+        const int exp = _scale_effect(7500 * you.experience_level * pow / 40, is_potion);
         if (you.has_mutation(MUT_DISTRIBUTED_TRAINING))
         {
             you.exp_available += exp;
@@ -525,25 +592,26 @@ public:
         static PotionMagic inst; return inst;
     }
 
-    bool can_quaff(string *reason = nullptr) const override
+    bool can_quaff(string *reason = nullptr, bool temp = true) const override
     {
-        if (you.magic_points == you.max_magic_points)
+        if (you.has_mutation(MUT_HP_CASTING) || temp && !you.max_magic_points)
         {
             if (reason)
-            {
-                if (you.max_magic_points)
-                    *reason = "Your magic is already full.";
-                else
-                    *reason = "You have no magic to restore.";
-            }
+                *reason = "You have no magic to restore.";
+            return false;
+        }
+        else if (temp && you.magic_points == you.max_magic_points)
+        {
+            if (reason)
+                *reason = "Your magic is already full.";
             return false;
         }
         return true;
     }
 
-    bool effect(bool=true, int = 40, bool=true) const override
+    bool effect(bool=true, int = 40, bool is_potion=true) const override
     {
-        inc_mp(POT_MAGIC_MP);
+        inc_mp(_scale_effect(POT_MAGIC_MP, is_potion));
         if (you.has_mutation(MUT_HP_CASTING))
             mpr("Magic washes over you without effect.");
         else
@@ -563,9 +631,9 @@ public:
         static PotionBerserk inst; return inst;
     }
 
-    bool can_quaff(string *reason = nullptr) const override
+    bool can_quaff(string *reason = nullptr, bool temp = true) const override
     {
-        return you.can_go_berserk(true, true, true, reason);
+        return you.can_go_berserk(true, true, true, reason, temp);
     }
 
     bool effect(bool was_known = true, int = 40, bool=true) const override
@@ -602,9 +670,9 @@ public:
  *               to mutate
  * @returns true if the player is able to mutate right now, otherwise false.
  */
-static bool _can_mutate(string *reason)
+static bool _can_mutate(string *reason, bool temp)
 {
-    if (you.can_safely_mutate())
+    if (you.can_safely_mutate(temp))
         return true;
 
     if (reason)
@@ -626,10 +694,11 @@ public:
         static PotionResistance inst; return inst;
     }
 
-    bool effect(bool=true, int pow = 40, bool=true) const override
+    bool effect(bool=true, int pow = 40, bool is_potion=true) const override
     {
         mprf(MSGCH_DURATION, "You feel protected.");
-        you.increase_duration(DUR_RESISTANCE, random2(pow) + 35);
+        const int add = _scale_effect(random2(pow) + 35, is_potion);
+        you.increase_duration(DUR_RESISTANCE, add);
         return true;
     }
 };
@@ -645,14 +714,29 @@ public:
         static PotionLignify inst; return inst;
     }
 
-    bool can_quaff(string *reason = nullptr) const override
+    bool can_quaff(string *reason = nullptr, bool temp = true) const override
     {
-        return transform(0, transformation::tree, false, true, reason);
+        if (you.is_lifeless_undead(temp))
+        {
+            if (reason)
+            {
+                if (!temp || you.is_lifeless_undead(false))
+                    *reason = "Your unliving flesh cannot be transformed in this way.";
+                else
+                    *reason = "You cannot currently transmute yourself.";
+            }
+            return false;
+        }
+        // transformation prevented for other reasons than lifeless undead.
+        // These should all be temp reasons (e.g. in an uncancellable form)...
+        if (temp)
+            return transform(0, transformation::tree, false, true, reason);
+        return true;
     }
 
-    bool effect(bool was_known = true, int=40, bool=true) const override
+    bool effect(bool was_known = true, int=40, bool is_potion=true) const override
     {
-        return transform(30, transformation::tree, !was_known);
+        return transform(_scale_effect(30, is_potion), transformation::tree, !was_known);
     }
 
     bool quaff(bool was_known) const override
@@ -666,7 +750,7 @@ public:
             }
 
             const cloud_type cloud = cloud_type_at(you.pos());
-            if (is_damaging_cloud(cloud, false)
+            if (cloud_damages_over_time(cloud, false)
                 // Tree form is immune to these two.
                 && cloud != CLOUD_MEPHITIC && cloud != CLOUD_POISON
                 && !yesno(make_stringf("Really become a tree while standing in "
@@ -706,23 +790,21 @@ public:
         static PotionMutation inst; return inst;
     }
 
-    bool can_quaff(string *reason = nullptr) const override
+    bool can_quaff(string *reason = nullptr, bool temp = true) const override
     {
-        if (!_can_mutate(reason))
-            return false;
-
-        return true;
+        return _can_mutate(reason, temp);
     }
 
-    bool effect(bool = true, int = 40, bool = true) const override
+    bool effect(bool = true, int = 40, bool is_potion= true) const override
     {
         if (have_passive(passive_t::cleanse_mut_potions))
             simple_god_message(" cleanses your potion of mutation!");
         else
             mpr("You feel extremely strange.");
         bool mutated = false;
-        int remove_mutations = random_range(MIN_REMOVED, MAX_REMOVED);
-        int add_mutations = random_range(MIN_ADDED, MAX_ADDED);
+        int remove_mutations = _scale_effect(random_range(MIN_REMOVED, MAX_REMOVED),
+                                             is_potion);
+        int add_mutations = _scale_effect(random_range(MIN_ADDED, MAX_ADDED), is_potion);
 
         // Remove mutations.
         for (int i = 0; i < remove_mutations; i++)
@@ -733,7 +815,7 @@ public:
         for (int i = 0; i < add_mutations; i++)
             mutated |= mutate(RANDOM_MUTATION, "potion of mutation", false);
         // Sometimes one good mutation.
-        if (coinflip())
+        if (coinflip() || is_potion && you.has_mutation(MUT_LONG_TONGUE))
         {
             mutated |= mutate(RANDOM_GOOD_MUTATION, "potion of mutation",
                               false);
@@ -780,13 +862,18 @@ public:
         static PotionDegeneration inst; return inst;
     }
 
-    bool effect(bool=true, int=40, bool=true) const override
+    bool effect(bool=true, int=40, bool is_potion=true) const override
     {
         mpr("There was something very wrong with that liquid.");
         bool success = false;
         for (int i = 0; i < NUM_STATS; ++i)
-            if (lose_stat(static_cast<stat_type>(i), 1 + random2(3)))
+        {
+            if (lose_stat(static_cast<stat_type>(i),
+                          _scale_effect(1 + random2(3), is_potion)))
+            {
                 success = true;
+            }
+        }
         return success;
     }
 
