@@ -144,9 +144,6 @@ int artefact_value(const item_def &item)
     if (prop[ARTP_ANGRY])
         ret -= 3;
 
-    if (prop[ARTP_CAUSE_TELEPORTATION])
-        ret -= 3;
-
     if (prop[ARTP_NOISE])
         ret -= 5;
 
@@ -232,14 +229,11 @@ unsigned int item_value(item_def item, bool ident)
             case SPWPN_DRAINING:
             case SPWPN_FLAMING:
             case SPWPN_FREEZING:
+            case SPWPN_HEAVY:
             case SPWPN_HOLY_WRATH:
                 valued *= 18;
                 break;
 
-            case SPWPN_VORPAL:
-                valued *= 15;
-                break;
-
             case SPWPN_PROTECTION:
             case SPWPN_VENOM:
                 valued *= 12;
@@ -281,31 +275,26 @@ unsigned int item_value(item_def item, bool ident)
             {
             case SPMSL_NORMAL:
             default:
-                valued *= 10;
                 break;
 
             case SPMSL_CHAOS:
-                valued *= 40;
+            case SPMSL_CURARE:
+                valued *= 4;
                 break;
 
-            case SPMSL_CURARE:
             case SPMSL_BLINDING:
+            case SPMSL_FRENZY:
             case SPMSL_SILVER:
+            case SPMSL_DISPERSAL:
 #if TAG_MAJOR_VERSION == 34
             case SPMSL_PARALYSIS:
             case SPMSL_PENETRATION:
             case SPMSL_STEEL:
-#endif
-            case SPMSL_DISPERSAL:
-                valued *= 30;
-                break;
-
-#if TAG_MAJOR_VERSION == 34
             case SPMSL_FLAME:
             case SPMSL_FROST:
             case SPMSL_SLEEP:
             case SPMSL_CONFUSION:
-                valued *= 25;
+                valued *= 3;
                 break;
 #endif
 
@@ -316,12 +305,9 @@ unsigned int item_value(item_def item, bool ident)
             case SPMSL_SLOW:
             case SPMSL_SICKNESS:
 #endif
-            case SPMSL_FRENZY:
-                valued *= 20;
+                valued *= 2;
                 break;
             }
-
-            valued /= 10;
         }
         break;
 
@@ -558,9 +544,11 @@ unsigned int item_value(item_def item, bool ident)
                     base = 3 * item.plus;
                     break;
                 case RING_PROTECTION:
-                case RING_EVASION:
                     base = 2 * item.plus;
                     break;
+                case RING_EVASION:
+                    base = 8 * item.plus / 5;
+                    break;
                 case RING_STRENGTH:
                 case RING_DEXTERITY:
                 case RING_INTELLIGENCE:
@@ -721,18 +709,7 @@ bool is_worthless_consumable(const item_def &item)
         CASE_REMOVED_POTIONS(item.sub_type)
         }
     case OBJ_SCROLLS:
-        switch (item.sub_type)
-        {
-#if TAG_MAJOR_VERSION == 34
-        case SCR_CURSE_ARMOUR:
-        case SCR_CURSE_WEAPON:
-        case SCR_CURSE_JEWELLERY:
-#endif
-        case SCR_NOISE:
-            return true;
-        default:
-            return false;
-        }
+        return item.sub_type == SCR_NOISE;
 
     // Only consumables are worthless.
     default:
@@ -888,7 +865,7 @@ class ShopEntry : public InvEntry
                                                   YELLOW;
         const string keystr = colour_to_str(keycol);
         const string itemstr =
-            colour_to_str(menu_colour(text, item_prefix(*item), tag));
+            colour_to_str(menu_colour(text, item_prefix(*item, false), tag, false));
         return make_stringf(" <%s>%c %c </%s><%s>%4d gold   %s%s</%s>",
                             keystr.c_str(),
                             hotkeys[0],
@@ -1058,6 +1035,8 @@ void ShopMenu::update_help()
 
 void ShopMenu::purchase_selected()
 {
+    no_excursions sanity;
+
     bool buying_from_list = false;
     vector<MenuEntry*> selected = selected_entries();
     int cost = selected_cost();
@@ -1389,6 +1368,8 @@ void shop()
     // If the shop is now empty, erase it from the overview.
     if (shop.stock.empty())
         destroy_shop_at(you.pos());
+    // finally it is safe to catch up on any off-level id stuff that is needed
+    shopping_list.do_excursion_work();
     redraw_screen();
     update_screen();
     if (menu.bought_something)
@@ -1628,8 +1609,8 @@ bool ShoppingList::add_thing(const item_def &item, int cost,
 
     if (!find_thing(item, pos).empty()) // TODO: this check isn't working?
     {
-        mprf(MSGCH_ERROR, "%s is already on the shopping list.",
-             item.name(DESC_THE).c_str());
+        ui::error(make_stringf("%s is already on the shopping list.",
+             item.name(DESC_THE).c_str()));
         return false;
     }
 
@@ -1698,8 +1679,8 @@ bool ShoppingList::del_thing(const item_def &item,
 
     if (indices.empty())
     {
-        mprf(MSGCH_ERROR, "%s isn't on shopping list, can't delete it.",
-             item.name(DESC_THE).c_str());
+        ui::error(make_stringf("%s isn't on shopping list, can't delete it.",
+             item.name(DESC_THE).c_str()));
         return false;
     }
 
@@ -1715,8 +1696,8 @@ bool ShoppingList::del_thing(string desc, const level_pos* _pos)
 
     if (indices.empty())
     {
-        mprf(MSGCH_ERROR, "%s isn't on shopping list, can't delete it.",
-             desc.c_str());
+        ui::error(make_stringf("%s isn't on shopping list, can't delete it.",
+             desc.c_str()));
         return false;
     }
 
@@ -1896,6 +1877,17 @@ bool ShoppingList::cull_identical_items(const item_def& item, int cost)
     return to_del.size();
 }
 
+void ShoppingList::do_excursion_work()
+{
+    ASSERT(level_excursions_allowed());
+    // this is not (currently) automatically called, so be sure to call it
+    // manually if you trigger item_type_identified at a time where
+    // excursions are disabled. (XX autocall every once in a while?)
+    for (auto &p : need_excursions)
+        item_type_identified(p.first, p.second);
+    need_excursions.clear();
+}
+
 void ShoppingList::item_type_identified(object_class_type base_type,
                                         int sub_type)
 {
@@ -1903,6 +1895,12 @@ void ShoppingList::item_type_identified(object_class_type base_type,
     if (!crawl_state.need_save)
         return;
 
+    if (!level_excursions_allowed())
+    {
+        need_excursions.emplace_back(base_type, sub_type);
+        return;
+    }
+
     // Only restore the excursion at the very end.
     level_excursion le;
 
@@ -2224,9 +2222,9 @@ void ShoppingList::fill_out_menu(Menu& shopmenu)
             // Colour shopping list item according to menu colours.
             const item_def &item = get_thing_item(thing);
 
-            const string colprf = item_prefix(item);
+            const string colprf = item_prefix(item, false);
             const int col = menu_colour(item.name(DESC_A),
-                                        colprf, "shop");
+                                        colprf, "shop", false);
 
             vector<tile_def> item_tiles;
             get_tiles_for_item(item, item_tiles, true);
@@ -2316,8 +2314,7 @@ void ShoppingList::display(bool view_only)
             const int index = shopmenu.get_entry_index(&sel);
             if (index == -1)
             {
-                mprf(MSGCH_ERROR, "ERROR: Unable to delete thing from shopping list!");
-                more();
+                ui::error("ERROR: Unable to delete thing from shopping list!");
                 return true;
             }
 
