@@ -235,8 +235,11 @@ static unordered_set<int> _player_nonbook_spells =
     SPELL_SONIC_WAVE,
     // religion
     SPELL_SMITING,
+    SPELL_MINOR_DESTRUCTION,
     // Ds powers
     SPELL_HURL_DAMNATION,
+    // Green Draconian breath
+    SPELL_MEPHITIC_BREATH,
 };
 
 bool is_player_spell(spell_type which_spell)
@@ -279,6 +282,13 @@ static void _list_available_spells(spell_set &available_spells)
         available_spells.insert(gift);
 }
 
+static bool _spell_available_to_memorize(spell_type which_spell)
+{
+    spell_set available_spells;
+    _list_available_spells(available_spells);
+    return available_spells.count(which_spell) > 0;
+}
+
 bool player_has_available_spells()
 {
     spell_set available_spells;
@@ -1005,6 +1015,12 @@ static bool _learn_spell_checks(spell_type specspell, bool wizard = false)
         return false;
     }
 
+    if (!wizard && !_spell_available_to_memorize(specspell))
+    {
+        mpr("You haven't found that spell!");
+        return false;
+    }
+
     return true;
 }
 
@@ -1081,6 +1097,7 @@ bool book_has_title(const item_def &book)
     if (book.sub_type == BOOK_BIOGRAPHIES_II
         || book.sub_type == BOOK_BIOGRAPHIES_VII
         || book.sub_type == BOOK_OZOCUBU
+        || book.sub_type == BOOK_MAXWELL
         || book.sub_type == BOOK_UNRESTRAINED)
     {
         return true;
