@@ -408,48 +408,64 @@ static bool _check_fall_down_stairs(const dungeon_feature_type ftype, bool going
 
 static void _rune_effect(dungeon_feature_type ftype)
 {
-    // Nothing even remotely flashy for Zig.
-    if (ftype != DNGN_ENTER_ZIGGURAT)
+    vector<int> runes;
+    for (int i = 0; i < NUM_RUNE_TYPES; i++)
+        if (you.runes[i])
+            runes.push_back(i);
+
+    ASSERT(runes.size() >= 1);
+    shuffle_array(runes);
+
+    // Zot is extra flashy.
+    if (ftype == DNGN_ENTER_ZOT)
     {
-        vector<int> runes;
-        for (int i = 0; i < NUM_RUNE_TYPES; i++)
-            if (you.runes[i])
-                runes.push_back(i);
+        ASSERT(runes.size() >= 3);
 
-        ASSERT(runes.size() >= 1);
-        shuffle_array(runes);
-
-        // Zot is extra flashy.
-        if (ftype == DNGN_ENTER_ZOT)
-        {
-            ASSERT(runes.size() >= 3);
-
-            mprf("You insert the %s rune into the lock.", rune_type_name(runes[2]));
+        mprf("You insert the %s rune into the lock.", rune_type_name(runes[2]));
 #ifdef USE_TILE_LOCAL
-            view_add_tile_overlay(you.pos(), tileidx_zap(rune_colour(runes[2])));
-            viewwindow(false);
-            update_screen();
+        view_add_tile_overlay(you.pos(), tileidx_zap(rune_colour(runes[2])));
+        viewwindow(false);
+        update_screen();
 #else
-            flash_view(UA_BRANCH_ENTRY, rune_colour(runes[2]));
+        flash_view(UA_BRANCH_ENTRY, rune_colour(runes[2]));
 #endif
-            mpr("The lock glows eerily!");
-            // included in default force_more_message
+        mpr("The lock glows eerily!");
+        // included in default force_more_message
 
-            mprf("You insert the %s rune into the lock.", rune_type_name(runes[1]));
-            big_cloud(CLOUD_BLUE_SMOKE, &you, you.pos(), 20, 7 + random2(7));
-            viewwindow();
-            update_screen();
-            mpr("Heavy smoke blows from the lock!");
-            // included in default force_more_message
+        mprf("You insert the %s rune into the lock.", rune_type_name(runes[1]));
+        big_cloud(CLOUD_BLUE_SMOKE, &you, you.pos(), 20, 7 + random2(7));
+        viewwindow();
+        update_screen();
+        mpr("Heavy smoke blows from the lock!");
+        // included in default force_more_message
+    }
+
+    mprf("You insert the %s rune into the lock.", rune_type_name(runes[0]));
+
+    if (silenced(you.pos()))
+        mpr("The gate opens wide!");
+    else
+        mpr("With a soft hiss the gate opens wide!");
+    // these are included in default force_more_message
+}
+
+static void _maybe_use_runes(dungeon_feature_type ftype)
+{
+    switch (ftype)
+    {
+    case DNGN_ENTER_ZOT:
+        if (!you.level_visited(level_id(BRANCH_ZOT, 1)))
+            _rune_effect(ftype);
+        break;
+    case DNGN_EXIT_VAULTS:
+        if (vaults_is_locked())
+        {
+            unlock_vaults();
+            _rune_effect(ftype);
         }
-
-        mprf("You insert the %s rune into the lock.", rune_type_name(runes[0]));
-
-        if (silenced(you.pos()))
-            mpr("The gate opens wide!");
-        else
-            mpr("With a soft hiss the gate opens wide!");
-        // these are included in default force_more_message
+        break;
+    default:
+        break;
     }
 }
 
@@ -561,7 +577,7 @@ static level_id _travel_destination(const dungeon_feature_type how,
     level_id dest;
     if (shaft)
     {
-        if (!is_valid_shaft_level())
+        if (!is_valid_shaft_level(false))
         {
             if (known_shaft)
                 mpr("The shaft disappears in a puff of logic!");
@@ -638,19 +654,7 @@ static level_id _travel_destination(const dungeon_feature_type how,
 
     // Maybe perform the entry sequence (we check that they have enough runes
     // in main.cc: _can_take_stairs())
-    for (branch_iterator it; it; ++it)
-    {
-        if (how != it->entry_stairs)
-            continue;
-
-        if (!you.level_visited(level_id(it->id, 1))
-            && runes_for_branch(it->id) > 0)
-        {
-            _rune_effect(how);
-        }
-
-        break;
-    }
+    _maybe_use_runes(how);
 
     // Markers might be deleted when removing portals.
     const string dst = env.markers.property_at(you.pos(), MAT_ANY, "dst");
@@ -786,8 +790,7 @@ void floor_transition(dungeon_feature_type how,
         mark_milestone("abyss.enter", "escaped (hah) into the Abyss!");
         take_note(Note(NOTE_MESSAGE, 0, 0, "Took an exit into the Abyss."), true);
     }
-    else if (how == DNGN_EXIT_ABYSS
-             && you.chapter != CHAPTER_POCKET_ABYSS)
+    else if (how == DNGN_EXIT_ABYSS)
     {
         mark_milestone("abyss.exit", "escaped from the Abyss!");
         you.attribute[ATTR_BANISHMENT_IMMUNITY] = you.elapsed_time + 100
@@ -947,6 +950,12 @@ void floor_transition(dungeon_feature_type how,
                 update_vision_range();
         }
 
+        if (how == DNGN_ENTER_VAULTS && !runes_in_pack())
+        {
+            lock_vaults();
+            mpr("The door slams shut behind you.");
+        }
+
         if (branch == BRANCH_GAUNTLET)
             _gauntlet_effect();
 
@@ -1207,8 +1216,6 @@ level_id stair_destination(dungeon_feature_type feat, const string &dst,
         return level_id(BRANCH_VESTIBULE);
 
     case DNGN_EXIT_ABYSS:
-        if (you.chapter == CHAPTER_POCKET_ABYSS)
-            return level_id(BRANCH_DUNGEON, 1);
 #if TAG_MAJOR_VERSION == 34
     case DNGN_EXIT_PORTAL_VAULT:
 #endif
@@ -1245,7 +1252,6 @@ level_id stair_destination(dungeon_feature_type feat, const string &dst,
     return level_id();
 }
 
-// TODO(Zannick): Fully merge with up_stairs into take_stairs.
 void down_stairs(dungeon_feature_type force_stair, bool force_known_shaft, bool update_travel_cache)
 {
     take_stairs(force_stair, false, force_known_shaft, update_travel_cache);
