@@ -275,6 +275,11 @@ enum tag_minor_version
     TAG_MINOR_MERGE_RANGED,        // Merge all ranged weapon skills together.
     TAG_MINOR_RECOMPRESS_BADMUTS,  // Reduce some more mutations to 2 levels.
     TAG_MINOR_SPAWN_RATE,          // Remove the env.spawn_random_rate field.
+    TAG_MINOR_REMOVE_AK,           // Remove Abyssal Knight.
+    TAG_MINOR_BUTTERSUMMONS,       // Alternate ?butt with ?summ, not ?fog.
+    TAG_MINOR_WU_ABILITIES,        // Make Lunge and Whirlwind Abil, not Invok
+    TAG_MINOR_MORE_WAYPOINTS,      // Increase the number of allowed waypoints.
+    TAG_MINOR_GENERATED_MISC,      // Track generated misc item types.
 #endif
     NUM_TAG_MINORS,
     TAG_MINOR_VERSION = NUM_TAG_MINORS - 1
