@@ -12,6 +12,7 @@
 #include <cmath>
 
 #include "areas.h"
+#include "art-enum.h"
 #include "bloodspatter.h"
 #include "branch.h"
 #include "cloud.h"
@@ -50,6 +51,15 @@
 #include "travel.h"
 #include "xom.h"
 
+static string _net_immune_reason()
+{
+    if (player_equip_unrand(UNRAND_SLICK_SLIPPERS))
+        return "You slip through the net.";
+    if (you.body_size(PSIZE_BODY) >= SIZE_GIANT)
+        return "The net is torn apart by your bulk.";
+    return "";
+}
+
 static const string TRAP_PROJECTILE_KEY = "trap_projectile";
 
 bool trap_def::active() const
@@ -155,6 +165,9 @@ bool trap_def::is_safe(actor* act) const
     if (type == TRAP_GOLUBRIA || type == TRAP_SHAFT)
         return true;
 
+    if (type == TRAP_NET && !_net_immune_reason().empty())
+        return true;
+
     // Let players specify traps as safe via lua.
     if (clua.callbooleanfn(false, "c_trap_is_safe", "s", trap_name(type).c_str()))
         return true;
@@ -258,7 +271,18 @@ bool monster_caught_in_net(monster* mon)
         return false;
     }
 
-    if (mons_class_is_stationary(mon->type))
+    if (mon->is_insubstantial())
+    {
+        if (you.can_see(*mon))
+        {
+            mprf("The net passes right through %s!",
+                 mon->name(DESC_THE).c_str());
+        }
+        return false;
+    }
+
+    monster_info mi(mon);
+    if (mi.net_immune())
     {
         if (you.see_cell(mon->pos()))
         {
@@ -273,16 +297,6 @@ bool monster_caught_in_net(monster* mon)
         return false;
     }
 
-    if (mon->is_insubstantial())
-    {
-        if (you.can_see(*mon))
-        {
-            mprf("The net passes right through %s!",
-                 mon->name(DESC_THE).c_str());
-        }
-        return false;
-    }
-
     if (!mon->caught() && mon->add_ench(ENCH_HELD))
     {
         if (you.see_cell(mon->pos()))
@@ -300,23 +314,22 @@ bool monster_caught_in_net(monster* mon)
 
 bool player_caught_in_net()
 {
-    if (you.body_size(PSIZE_BODY) >= SIZE_GIANT)
+    if (!_net_immune_reason().empty())
         return false;
 
-    if (!you.attribute[ATTR_HELD])
-    {
-        mpr("You become entangled in the net!");
-        stop_running();
+    if (you.attribute[ATTR_HELD])
+        return false;
 
-        // Set the attribute after the mpr, otherwise the screen updates
-        // and we get a glimpse of a web because there isn't a trapping net
-        // item yet
-        you.attribute[ATTR_HELD] = 1;
+    mpr("You become entangled in the net!");
+    stop_running();
 
-        stop_delay(true); // even stair delays
-        return true;
-    }
-    return false;
+    // Set the attribute after the mpr, otherwise the screen updates
+    // and we get a glimpse of a web because there isn't a trapping net
+    // item yet
+    you.attribute[ATTR_HELD] = 1;
+
+    stop_delay(true); // even stair delays
+    return true;
 }
 
 void check_net_will_hold_monster(monster* mons)
@@ -627,7 +640,11 @@ void trap_def::trigger(actor& triggerer)
 
         if (!player_caught_in_net())
         {
-            mpr("The net is torn apart by your bulk.");
+            if (!m) // no message already printed
+                mpr("You trigger the net trap.");
+            const string reason = _net_immune_reason();
+            if (!reason.empty())
+                mprf("%s", reason.c_str());
             break;
         }
 
@@ -1022,9 +1039,10 @@ dungeon_feature_type trap_feature(trap_type type)
 /***
  * Can a shaft be placed on the current level?
  *
+ * @param respect_brflags Whether brflag::no_shafts should be factored in.
  * @returns true if such a shaft can be placed.
  */
-bool is_valid_shaft_level()
+bool is_valid_shaft_level(bool respect_brflags)
 {
     // Important: We are sometimes called before the level has been loaded
     // or generated, so should not depend on properties of the level itself,
@@ -1038,19 +1056,24 @@ bool is_valid_shaft_level()
 
     const Branch &branch = branches[place.branch];
 
-    if (branch.branch_flags & brflag::no_shafts)
+    if (respect_brflags && branch.branch_flags & brflag::no_shafts)
         return false;
 
     // Don't allow shafts from the bottom of a branch.
     return (brdepth[place.branch] - place.depth) >= 1;
 }
 
-///
 static bool& _shafted_in(const Branch &branch)
 {
     return you.props[make_stringf("shafted_in_%s", branch.abbrevname)].get_bool();
 }
 
+/// Mark the player as having been shafted in the current branch.
+void set_shafted()
+{
+    _shafted_in(branches[you.where_are_you]) = true;
+}
+
 /**
  * Can we force shaft the player from this level?
  *
@@ -1076,7 +1099,7 @@ void roll_trap_effects()
 {
     int trap_rate = trap_rate_for_place();
 
-    you.trapped = you.num_turns && !have_passive(passive_t::avoid_traps)
+    you.trapped = you.num_turns
         && env.density > 0 // can happen with builder in debug state
         && (you.trapped || x_chance_in_y(trap_rate, 9 * env.density));
 }
@@ -1119,8 +1142,13 @@ void do_trap_effects()
         case TRAP_SHAFT:
             dprf("Attempting to shaft player.");
             _print_malev();
+            if (have_passive(passive_t::avoid_traps))
+            {
+                simple_god_message(" reveals a hidden shaft just before you would have fallen in.");
+                return;
+            }
             if (you.do_shaft(false))
-                _shafted_in(branches[you.where_are_you]) = true;
+                set_shafted();
             break;
 
         case TRAP_ALARM:
@@ -1129,6 +1157,11 @@ void do_trap_effects()
             // XXX: improve messaging to make it clear there's a wail outside of the
             // player's silence
             _print_malev();
+            if (have_passive(passive_t::avoid_traps))
+            {
+                simple_god_message(" reveals an alarm trap just before you would have tripped it.");
+                return;
+            }
             mprf("With a horrendous wail, an alarm goes off!");
             fake_noisy(40, you.pos());
             you.sentinel_mark(true);
@@ -1138,6 +1171,12 @@ void do_trap_effects()
         {
             string msg = make_stringf("%s and a teleportation trap spontaneously manifests!",
                                       _malev_msg().c_str());
+            if (have_passive(passive_t::avoid_traps))
+            {
+                mprf("%s", msg.c_str());
+                simple_god_message(" warns you in time for you to avoid it.");
+                return;
+            }
             you_teleport_now(false, true, msg);
             break;
         }
