@@ -38,7 +38,8 @@ void destroy_trap(const coord_def& pos);
 trap_def* trap_at(const coord_def& where);
 trap_type get_trap_type(const coord_def& where);
 
-bool is_valid_shaft_level();
+bool is_valid_shaft_level(bool respect_brflags = true);
+void set_shafted();
 void roll_trap_effects();
 void do_trap_effects();
 level_id generic_shaft_dest(level_id place);
