@@ -16,12 +16,16 @@ import tornado.ioloop
 import tornado.template
 import tornado.web
 from tornado.ioloop import IOLoop
+# do not add tornado.platform here without changing do_chroot()
 
 import webtiles
 from webtiles import auth, load_games, process_handler, userdb, config
-from webtiles import game_data_handler, util, ws_handler
+from webtiles import game_data_handler, util, ws_handler, status
 
 class MainHandler(tornado.web.RequestHandler):
+    # async def _execute(self, transforms, *args, **kwargs):
+    #     await tornado.web.RequestHandler._execute(self, transforms, *args, **kwargs)
+
     def get(self):
         host = self.request.host
         if self.request.protocol == "https" or self.request.headers.get("x-forwarded-proto") == "https":
@@ -38,7 +42,8 @@ class MainHandler(tornado.web.RequestHandler):
             if recovery_token_error:
                 logging.warning("Recovery token error from %s", self.request.remote_ip)
 
-        self.render("client.html", socket_server = protocol + host + "/socket",
+        with util.SlowWarning("Slow IO: render client.html"):
+            self.render("client.html", socket_server = protocol + host + "/socket",
                     username = None,
                     config = config,
                     reset_token = recovery_token, reset_token_error = recovery_token_error)
@@ -49,8 +54,41 @@ class NoCacheHandler(tornado.web.StaticFileHandler):
         self.set_header("Pragma", "no-cache")
         self.set_header("Expires", "0")
 
+
+_crawl_version = "unknown"
+
+def load_version():
+    global _crawl_version
+    try:
+        # this is the "bad" way to do this. However, the supposedly good ways
+        # to do this are all insanely complicated in ways that are irrelevant
+        # to webtiles.
+        with open(os.path.join(os.path.dirname(__file__), 'version.txt')) as f:
+            _crawl_version = f.readline().strip()
+    except:
+        _crawl_version = "unknown"
+
+
+def version_data():
+    return dict(
+        webtiles=_crawl_version,
+        tornado=tornado.version,
+        python="%d.%d.%d" % (sys.version_info[0], sys.version_info[1], sys.version_info[2]))
+
+
+def version():
+    vdata = version_data()
+    # TODO convert to f string some day...
+    return "Webtiles (%s) running with Tornado %s and Python %s" % (
+        vdata["webtiles"],
+        vdata["tornado"],
+        vdata["python"])
+
+
 def err_exit(errmsg, exc_info=False):
-    logging.error(errmsg, exc_info=exc_info)
+    if exc_info or config.get('logging_config').get('filename'):
+        # don't print duplicate messages on stdout
+        logging.error(errmsg, exc_info=exc_info)
     sys.exit(errmsg)
 
 def daemonize():
@@ -163,11 +201,16 @@ def bind_server():
     if config.get('no_cache', False):
         settings["static_handler_class"] = NoCacheHandler
 
-    application = tornado.web.Application([
+    handlers = [
             (r"/", MainHandler),
             (r"/socket", ws_handler.CrawlWebSocket),
-            (r"/gamedata/([0-9a-f]*\/.*)", game_data_handler.GameDataHandler)
-            ], gzip=config.get('use_gzip'), **settings)
+            (r"/gamedata/([0-9a-f]*\/.*)", game_data_handler.GameDataHandler),
+            (r"/status/lobby/", status.LobbyHandler),
+            (r"/status/version/", status.VersionHandler),
+            ]
+
+    application = tornado.web.Application(handlers,
+                                    gzip=config.get('use_gzip'), **settings)
 
     kwargs = {}
     if config.get('http_connection_timeout') is not None:
@@ -261,15 +304,18 @@ def monkeypatch_tornado24():
     IOLoop.current = staticmethod(IOLoop.instance)
 
 
-def ensure_tornado_current():
+def ensure_tornado_current(exit=True):
     try:
         tornado.ioloop.IOLoop.current()
     except AttributeError:
         monkeypatch_tornado24()
         tornado.ioloop.IOLoop.current()
-        logging.error(
-            "You are running a deprecated version of tornado; please update"
-            " to at least version 4.")
+        err = ("You are running a deprecated version of tornado; please update"
+               " to a current version.")
+        if exit:
+            sys.err(err)
+        else:
+            logging.error(err)
 
 
 def usr1_handler(signum, frame):
@@ -322,6 +368,10 @@ def parse_args_main():
 
 # override config with any arguments supplied on the command line
 def export_args_to_config(args):
+    if config.get('live_debug'):
+        config.server_config._load_override_file(os.path.join(
+                            config.get("server_path", ""), "debug-config.yml"))
+        config.do_early_logging() # sigh
     if args.port:
         config.set('bind_nonsecure', True)
         config.set('bind_address', "")  # TODO: ??
@@ -349,10 +399,11 @@ def reset_token_commands(args):
         username = args.reset
 
     user_info = userdb.get_user_info(username)
-
     if not user_info:
         err_exit("Reset/clear password failed; invalid user: %s" % username)
 
+    username = user_info.username # canonicalize
+
     # don't crash on the default config
     if config.get('lobby_url') is None:
         config.set('lobby_url', "[insert lobby url here]")
@@ -363,19 +414,108 @@ def reset_token_commands(args):
             err_exit("Error clearing password reset token for %s: %s" % (username, msg))
         else:
             print("Password reset token cleared for account '%s'." % username)
+            return True
     else:
         ok, msg = userdb.generate_forgot_password(username)
         if not ok:
             err_exit("Error generating password reset token for %s: %s" % (username, msg))
         else:
-            if not user_info[1]:
+            if not user_info.email:
                 logging.warning("No email set for account '%s', use caution!" % username)
             print("Setting a password reset token on account '%s'." % username)
-            print("Email: %s\nMessage body to send to user:\n%s\n" % (user_info[1], msg))
+            print("Email: %s\nMessage body to send to user:\n%s\n" % (user_info.email, msg))
+            return True
+    return False
+
+
+def mode_to_flag(mode):
+    if mode == "admin":
+        return userdb.DGLACCT_ADMIN
+    elif mode == "ban":
+        return userdb.DGLACCT_LOGIN_LOCK
+    elif mode == "hold":
+        return userdb.DGLACCT_ACCOUNT_HOLD
+    elif mode == "wizard":
+        if not config.get('wizard_accounts'):
+            logging.warning("Wizard accounts not enabled on this server, do you want `admin`?")
+        return userdb.DGLACCT_WIZARD
+    elif mode == "bot":
+        if not config.get('bot_accounts'):
+            logging.warning("Bot accounts are not enabled on this server...")
+        return userdb.DGLACCT_BOT
+    return 0
+
+
+def show_flags(username):
+    r = userdb.get_user_info(username)
+    if not r:
+        err_exit("Unknown user '%s'!" % username)
+    # XX would be nice to normalize username
+    if not r.flags:
+        print("User '%s' (id %d) has no flags set." % (r.username, r.id))
+    else:
+        print("Flags for '%s' (id %d): %s" % (r.username, r.id, userdb.flag_description(r.flags)))
+    return True
+
+
+def flag_commands(args):
+    if args.show:
+        return show_flags(args.show)
+
+    # everything below here needs an explicit flag
+    if not args.flag:
+        print("Missing flag to query!", file=sys.stderr)
+        return False
+    flag = mode_to_flag(args.flag)
+    if not flag:
+        print("Unknown flag '%s'" % args.list, file=sys.stderr)
+        return False
+
+    if args.list:
+        l = userdb.get_users_by_flag(flag)
+        if not l:
+            print("No matching users.")
+        else:
+            print("Users with flag '%s': %s" %
+                (userdb.flag_description(flag), ", ".join([u.username for u in l])))
+        return True
+    elif args.set:
+        r = userdb.set_flags(args.set, flag, mask=flag)
+        if r:
+            err_exit("Error when setting flag %s for '%s': %s" % (args.set, arg.flag, r))
+        else:
+            print("Flag set.")
+            show_flags(args.set)
+        return True
+    elif args.clear:
+        r = userdb.set_flags(args.clear, 0, mask=flag)
+        if r:
+            err_exit("Error when clearing flag %s for '%s': %s" % (args.clear, arg.flag, r))
+        else:
+            print("Flag cleared.")
+            show_flags(args.clear)
+        return True
+    return False
 
 
 def ban_commands(args):
-    if args.list or args.list_holds:
+    if args.check_config_bans or args.run_config_bans:
+        # potentially very heavy commands on old servers...
+        all_users = userdb.get_all_users()
+        affected = [n.username for n in all_users if not config.check_name(n.username)]
+        if not affected:
+            print("No affected users.")
+            return True
+        if args.check_config_bans:
+            print("%d existing user(s) have banned names:" % len(affected))
+        else:
+            print("Banning %d user(s)!" % len(affected))
+            for n in affected:
+                userdb.set_ban(n, True)
+        if affected:
+            print("    Affected users: %s" % ', '.join(affected))
+        return True
+    elif args.list or args.list_holds:
         banned, held = userdb.get_bans()
         if args.list:
             if len(banned):
@@ -386,6 +526,7 @@ def ban_commands(args):
             print("Account holds: " + ", ".join(held))
         else:
             print("No account holds!")
+        return True
     elif args.clear_holds:
         banned, held = userdb.get_bans()
         if len(held):
@@ -393,7 +534,7 @@ def ban_commands(args):
             for u in held:
                 err = userdb.set_account_hold(u, False)
                 if err:
-                    print(err, file=sys.stderr)
+                    err_exit("Error when clearing hold for '%s': %s" % (u, err))
                 else:
                     cleared.append(u)
             if len(cleared):
@@ -402,29 +543,33 @@ def ban_commands(args):
                 print("No holds cleared.")
         else:
             print("No holds to clear.")
+        return True
     elif args.add:
         err = userdb.set_ban(args.add, True)
         if err:
-            print(err, file=sys.stderr)
+            err_exit("Error when trying to ban user '%s': %s" % (args.add, err))
         else:
             print("'%s' is now banned." % args.add)
+        return True
     elif args.hold:
         err = userdb.set_account_hold(args.hold, True)
         if err:
-            print(err, file=sys.stderr)
+            err_exit("Error when trying to set hold for user '%s': %s" % (args.hold, err))
         else:
             print("Account hold set for '%s'." % args.hold)
+        return True
     elif args.clear:
-        # XX is there any use for fine-grained clear commands?
         err1 = userdb.set_account_hold(args.clear, False)
         if err1:
             err2 = userdb.set_ban(args.clear, False)
             if err2:
-                print(err2, file=sys.stderr)
+                err_exit("Error when trying to clear hold for user '%s': %s" % (args.clear, err2))
             else:
                 print("'%s' is no longer banned." % args.clear)
         else:
             print("Account hold cleared for '%s'." % args.clear)
+        return True
+    return False
 
 
 def parse_args_util():
@@ -432,6 +577,7 @@ def parse_args_util():
         description='Dungeon Crawl webtiles utilities.',
         epilog='Command line options will override config settings.')
     parser.add_argument('--logfile',
+        default='-',
         help='A logfile to write to; use "-" for stdout (the default, overrides config).')
     subparsers = parser.add_subparsers(help='Mode', dest='mode')
     subparsers.required = True
@@ -448,32 +594,65 @@ def parse_args_util():
     parser_ban.add_argument('--add', type=str, help='Set an account ban for a user. Replaces any account holds.')
     parser_ban.add_argument('--hold', type=str, help='Set an account hold for a user. Replaces any ban flags.')
     parser_ban.add_argument('--clear', type=str, help='Clear a ban/hold for a user.')
+    parser_ban.add_argument('--run-config-bans', action='store_true',
+        help='Ban any existing usernames disallowed by server config banned name options')
+    parser_ban.add_argument('--check-config-bans', action='store_true',
+        help='List any existing usernames that would be banned by --run-config-bans')
     parser_ban.add_argument('--list', action='store_true',
             help='List current banned users and account holds.')
     parser_ban.add_argument('--list-holds', action='store_true',
             help='List current account holds.')
     parser_ban.add_argument ('--clear-holds', action='store_true',
             help='Clear all current account holds')
+
+    parser_flag = subparsers.add_parser('flag',
+        help="Low level interface to query and set userdb flags.",
+        description="Low-level interface to query and set userdb flags. For managing bans/holds, the 'ban' command is recommended instead.")
+    parser_flag.add_argument('flag', type=str, nargs='?', default="", help='a flag to query with (ban, hold, admin, wizard, bot)')
+    parser_flag.add_argument('--list', action='store_true',
+        help='List any users with the indicated flag')
+    parser_flag.add_argument('--set', type=str, help='Set a flag on a specified user.')
+    parser_flag.add_argument('--clear', type=str, help='Clear a flag on a specified user.')
+    parser_flag.add_argument('--show', type=str, help='Show flags on a specified user (does not require [flag]).')
+
     result = parser.parse_args()
-    # XX is there a better way to do this
-    if result.mode == "password" and not result.reset and not result.clear_reset:
-        parser_pw.print_help()
-        sys.exit()
-    elif result.mode == "ban" and not result.add and not result.clear and not result.list and not result.hold and not result.clear_holds and not result.list_holds:
-        parser_ban.print_help()
-        sys.exit()
+    help_fun = parser.print_help
+    if result.mode == "password":
+        help_fun = parser_pw.print_help
+    elif result.mode == "ban":
+        help_fun = parser_ban.print_help
+    elif result.mode == "flag":
+        help_fun = parser_flag.print_help
 
     if not result.logfile:
         result.logfile = '-'
     result.daemon = False
     result.pidfile = False
-    return result
+    return result, help_fun
+
+
+def do_chroot():
+    if config.get('chroot'):
+        os.chroot(config.get('chroot'))
+        try:
+            # try to fail early, with an informative message, if this is not
+            # going to work
+            # the choice of tornado.platform is a bit heuristic, but it is
+            # currently where the webserver seems to fail on import after a chroot.
+            # If it is ever imported at the top of this file, something else would
+            # be needed.
+            import tornado.platform
+        except:
+            # no logging available yet
+            print("Error: can't import `tornado.platform` in chroot. Did you copy"
+                " the python library for this version of python into the chroot?",
+                file=sys.stderr)
+            raise
 
 
 def run_util():
-    args = parse_args_util()
-    if config.get('chroot'):
-        os.chroot(config.get('chroot'))
+    args, help_fun = parse_args_util()
+    do_chroot()
 
     if config.source_file is None:
         sys.exit("No configuration provided!")
@@ -483,29 +662,37 @@ def run_util():
     except:
         err_exit("Errors in config. Exiting.", exc_info=True)
 
-    # duplicate some minimal setup needed for this to work
-    config.get('logging_config').pop('filename', None)
-    args.logfile = "<stdout>"  # make the log message easier to read
+    # duplicate some minimal setup needed for this to work. Note that unlike
+    # the main server.py call, this arg parser sets '-' as the default for
+    # the log, overriding config.
+    if not args.logfile or args.logfile == "-":
+        config.get('logging_config').pop('filename', None)
+    else:
+        config.get('logging_config')['filename'] = args.logfile
 
     init_logging(config.get('logging_config'))
 
-    if config.get('dgl_mode'):
-        userdb.ensure_user_db_exists()
-        userdb.upgrade_user_db()
-    userdb.ensure_settings_db_exists()
+    userdb.init_db_connections()
+
+    mode_fun = None
 
     if args.mode == "password":
-        reset_token_commands(args)
+        mode_fun = reset_token_commands
     elif args.mode == "ban":
-        ban_commands(args)
+        mode_fun = ban_commands
+    elif args.mode == "flag":
+        mode_fun = flag_commands
+
+    if not mode_fun or not mode_fun(args):
+        help_fun()
+        sys.exit(1)
 
 
 # before running, this needs to have its config source set up. See
 # ../server.py in the official repository for an example.
 def run():
     args = parse_args_main()
-    if config.get('chroot'):
-        os.chroot(config.get('chroot'))
+    do_chroot()
 
     if config.source_file is None:
         # we could try to automatically figure this out from server_path, if
@@ -521,80 +708,97 @@ def run():
             config.get('logging_config')['filename'] = args.logfile
 
     init_logging(config.get('logging_config'))
-    logging.info("Loaded server configuration from: %s", config.source_file)
+    try:
+        logging.info("Loaded server configuration from: %s", config.source_file)
+        config.do_early_logging()
 
-    if config.get('live_debug'):
-        logging.info("Starting in live-debug mode.")
-        config.set('watch_socket_dirs', False)
+        if config.get('live_debug'):
+            logging.info("Starting in live-debug mode.")
+            config.set('watch_socket_dirs', False)
 
-    if args.logfile:
-        logging.info("Using command-line supplied logfile: '%s'", args.logfile)
+        if args.logfile:
+            logging.info("Using command-line supplied logfile: '%s'", args.logfile)
 
-    export_args_to_config(args)
+        export_args_to_config(args)
+
+        try:
+            config.load_game_data()
+            config.validate()
+        except:
+            err_exit("Errors in game data. Exiting.", exc_info=True)
+
+        if config.get('daemon', False):
+            daemonize()
+
+        signal.signal(signal.SIGTERM, signal_handler)
+        signal.signal(signal.SIGINT, signal_handler)
+        if (config.get('hup_reloads_config')):
+            signal.signal(signal.SIGHUP, reload_signal_handler)
+        else:
+            signal.signal(signal.SIGHUP, signal_handler)
+
+        if config.get('umask') is not None:
+            os.umask(config.get('umask'))
+    except SystemExit:
+        # logging already done, hopefully
+        raise
+    except:
+        err_exit("Server startup failed!", exc_info=True)
 
     try:
-        config.load_game_data()
-        config.validate()
+        write_pidfile()
+
+        global servers
+        servers = bind_server()
+        ensure_tornado_current()
+
+        shed_privileges()
+
+        # is this ever set to False by anyone in practice?
+        dgl_mode = config.get('dgl_mode')
+
+        userdb.init_db_connections()
+
+        signal.signal(signal.SIGUSR1, usr1_handler)
+
+        try:
+            IOLoop.current().set_blocking_log_threshold(0.5) # type: ignore
+            logging.info("Blocking call timeout: 500ms.")
+        except:
+            pass
+
+        if dgl_mode:
+            ws_handler.status_file_timeout() # note: tornado coroutine
+            auth.purge_login_tokens_timeout()
+            ws_handler.start_reading_milestones()
+
+            if config.get('watch_socket_dirs'):
+                process_handler.watch_socket_dirs()
+
+        # set up various timeout loops
+        ws_handler.do_periodic_lobby_updates()
+        webtiles.config.init_config_timeouts()
+
+        logging.info("DCSS Webtiles server started! (PID: %s)" % os.getpid())
+        logging.info(version())
+
+        IOLoop.current().start()
+
+        logging.info("Bye!")
+    except SystemExit:
+        # logging already done, hopefully
+        raise
     except:
-        err_exit("Errors in config. Exiting.", exc_info=True)
+        err_exit("Server startup failed!", exc_info=True)
+    finally:
+        # warning: need to be careful what appears in this finally block, since
+        # it may be called by child processes on fork in terminal.py in the
+        # event of rare bad timing or bugs. (So any global state that may be
+        # used here should be reset in the child process...)
+        remove_pidfile()
 
-    if config.get('daemon', False):
-        daemonize()
-
-    signal.signal(signal.SIGTERM, signal_handler)
-    signal.signal(signal.SIGINT, signal_handler)
-    if (config.get('hup_reloads_config')):
-        signal.signal(signal.SIGHUP, reload_signal_handler)
-    else:
-        signal.signal(signal.SIGHUP, signal_handler)
-
-    if config.get('umask') is not None:
-        os.umask(config.get('umask'))
-
-    write_pidfile()
-
-    global servers
-    servers = bind_server()
-    ensure_tornado_current()
-
-    shed_privileges()
-
-    # is this ever set to False by anyone in practice?
-    dgl_mode = config.get('dgl_mode')
-
-    if dgl_mode:
-        userdb.ensure_user_db_exists()
-        userdb.upgrade_user_db()
-    userdb.ensure_settings_db_exists()
-
-    signal.signal(signal.SIGUSR1, usr1_handler)
-
-    try:
-        IOLoop.current().set_blocking_log_threshold(0.5) # type: ignore
-        logging.info("Blocking call timeout: 500ms.")
-    except:
-        # this is the new normal; still not sure of a way to deal with this.
-        logging.info("Webserver running without a blocking call timeout.")
-
-    if dgl_mode:
-        ws_handler.status_file_timeout()
-        auth.purge_login_tokens_timeout()
-        ws_handler.start_reading_milestones()
-
-        if config.get('watch_socket_dirs'):
-            process_handler.watch_socket_dirs()
-
-    # start the lobby update timeout loop
-    ws_handler.do_periodic_lobby_updates()
-
-    logging.info("DCSS Webtiles server started with Tornado %s! (PID: %s)" %
-                                                (tornado.version, os.getpid()))
-
-    IOLoop.current().start()
-
-    logging.info("Bye!")
-    remove_pidfile()
 
+load_version()
 
 # TODO: it might be nice to simply make this module runnable, but that would
 # need some way of specifying the config source and `config.server_path`.
