@@ -399,6 +399,7 @@ void debug_stethoscope(int mon)
          "firing_pos=(%d,%d) patrol_point=(%d,%d) god=%s%s",
          (hab >= 0 && hab < NUM_HABITATS) ? ht_names[hab] : "INVALID",
          mons.asleep()                    ? "sleep"
+         : mons.behaviour == BEH_BATTY   ? "flitting"
          : mons_is_wandering(mons)       ? "wander"
          : mons_is_seeking(mons)         ? "seek"
          : mons_is_fleeing(mons)         ? "flee"
@@ -732,6 +733,12 @@ void wizard_move_player_or_monster(const coord_def& where)
     crawl_state.cancel_cmd_again();
     crawl_state.cancel_cmd_repeat();
 
+    if (!in_bounds(where))
+    {
+        mpr("Cannot move out of bounds.");
+        return;
+    }
+
     static bool already_moving = false;
 
     if (already_moving)
