@@ -364,10 +364,7 @@ void handle_wizard_command()
 void enter_explore_mode()
 {
     // WIZ_NEVER gives protection for those who have wiz compiles,
-    // and don't want to risk their characters. Also, and hackishly,
-    // it's used to prevent access for non-authorised users to wizard
-    // builds in dgamelaunch builds unless the game is started with the
-    // -wizard flag.
+    // and don't want to risk their characters.
     if (Options.explore_mode == WIZ_NEVER)
         return;
 
@@ -392,7 +389,8 @@ void enter_explore_mode()
         take_note(Note(NOTE_MESSAGE, 0, 0, "Entered explore mode."));
 
 #ifndef SCORE_WIZARD_CHARACTERS
-        _log_wizmode_entrance();
+        scorefile_entry se(INSTANT_DEATH, MID_NOBODY, KILLED_BY_EXPLORING, nullptr);
+        logfile_new_entry(se);
 #endif
 
         you.explore = true;
