@@ -21,7 +21,6 @@ struct zap_info
     dungeon_char_type glyph;
     bool can_beam;
     bool is_explosion;
-    int hit_loudness;
 }
 */
 
@@ -44,7 +43,6 @@ static zap_info _mon_hex_zap(zap_type ztype, beam_type beam,
         NUM_DCHAR_TYPES,
         false,
         false,
-        0
     };
 }
 
@@ -65,7 +63,6 @@ static const zap_info zap_data[] =
     DCHAR_FIRED_ZAP,
     true,
     false,
-    0
 },
 
 {
@@ -82,7 +79,6 @@ static const zap_info zap_data[] =
     DCHAR_FIRED_ZAP,
     true,
     false,
-    0
 },
 
 {
@@ -99,7 +95,6 @@ static const zap_info zap_data[] =
     DCHAR_FIRED_ZAP,
     true,
     false,
-    0
 },
 
 {
@@ -116,7 +111,6 @@ static const zap_info zap_data[] =
     DCHAR_FIRED_ZAP,
     true,
     false,
-    0
 },
 
 {
@@ -133,7 +127,6 @@ static const zap_info zap_data[] =
     DCHAR_FIRED_MISSILE,
     true,
     false,
-    0
 },
 
 {
@@ -150,7 +143,6 @@ static const zap_info zap_data[] =
     DCHAR_FIRED_MISSILE,
     false,
     false,
-    0
 },
 
 {
@@ -167,7 +159,6 @@ static const zap_info zap_data[] =
     DCHAR_FIRED_MISSILE,
     false,
     false,
-    0
 },
 
 {
@@ -184,7 +175,6 @@ static const zap_info zap_data[] =
     DCHAR_FIRED_MISSILE,
     false,
     false,
-    0
 },
 
 {
@@ -201,7 +191,6 @@ static const zap_info zap_data[] =
     DCHAR_FIRED_ZAP,
     false,
     true,
-    0
 },
 
 {
@@ -218,7 +207,6 @@ static const zap_info zap_data[] =
     DCHAR_FIRED_ZAP,
     true,
     false,
-    0
 },
 
 {
@@ -235,7 +223,6 @@ static const zap_info zap_data[] =
     DCHAR_FIRED_ZAP,
     true,
     false,
-    0
 },
 
 {
@@ -252,7 +239,6 @@ static const zap_info zap_data[] =
     DCHAR_FIRED_ZAP,
     true,
     false,
-    0
 },
 
 {
@@ -269,7 +255,22 @@ static const zap_info zap_data[] =
     DCHAR_FIRED_ZAP,
     false,
     true,
-    0 // Noise comes from explosion
+},
+
+{
+    ZAP_ICY_FLASH_FREEZE,
+    "icy flash freeze",
+    200,
+    nullptr,
+    nullptr,
+    new dicedef_calculator<3, 7, 1, 12>,
+    new tohit_calculator<5, 1, 3>,
+    WHITE,
+    false,
+    BEAM_ICE, // rC capped at 50%, use ZAP_FLASH_FREEZE if you don't want this
+    DCHAR_FIRED_ZAP,
+    false,
+    false,
 },
 
 {
@@ -282,11 +283,10 @@ static const zap_info zap_data[] =
     new tohit_calculator<5, 1, 3>,
     WHITE,
     false,
-    BEAM_ICE,
+    BEAM_COLD, // normal rC (e.g. fully resisted at rC+++)
     DCHAR_FIRED_ZAP,
     false,
     false,
-    0
 },
 
 {
@@ -303,7 +303,6 @@ static const zap_info zap_data[] =
     DCHAR_FIRED_ZAP,
     false,
     false,
-    0
 },
 
 {
@@ -320,7 +319,6 @@ static const zap_info zap_data[] =
     DCHAR_FIRED_ZAP,
     true,
     false,
-    0
 },
 
 {
@@ -337,7 +335,6 @@ static const zap_info zap_data[] =
     DCHAR_FIRED_ZAP,
     true,
     false,
-    0
 },
 
 {
@@ -354,7 +351,6 @@ static const zap_info zap_data[] =
     DCHAR_FIRED_ZAP,
     true,
     false,
-    0
 },
 
 {
@@ -371,7 +367,6 @@ static const zap_info zap_data[] =
     DCHAR_FIRED_ZAP,
     true,
     false,
-    0
 },
 
 {
@@ -388,7 +383,6 @@ static const zap_info zap_data[] =
     DCHAR_FIRED_ZAP,
     true,
     false,
-    0
 },
 
 {
@@ -405,7 +399,6 @@ static const zap_info zap_data[] =
     DCHAR_FIRED_ZAP,
     true,
     false,
-    0
 },
 
 {
@@ -422,7 +415,6 @@ static const zap_info zap_data[] =
     DCHAR_FIRED_ZAP,
     false,
     false,
-    2
 },
 
 {
@@ -439,7 +431,6 @@ static const zap_info zap_data[] =
     DCHAR_FIRED_ZAP,
     false,
     false,
-    2
 },
 
 {
@@ -447,7 +438,7 @@ static const zap_info zap_data[] =
     "",
     100,
     nullptr,
-    new tohit_calculator<0, 3, 2>,
+    nullptr,
     nullptr,
     new tohit_calculator<0, 1, 3>,
     BLACK,
@@ -456,7 +447,6 @@ static const zap_info zap_data[] =
     NUM_DCHAR_TYPES,
     false,
     false,
-    0
 },
 
 {
@@ -473,7 +463,6 @@ static const zap_info zap_data[] =
     DCHAR_FIRED_ZAP,
     false,
     true,
-    0 // Noise comes from explosion
 },
 
 {
@@ -490,7 +479,6 @@ static const zap_info zap_data[] =
     NUM_DCHAR_TYPES,
     false,
     false,
-    0
 },
 
 {
@@ -507,7 +495,6 @@ static const zap_info zap_data[] =
     DCHAR_FIRED_ZAP,
     false,
     false,
-    1
 },
 
 _mon_hex_zap(ZAP_PARALYSE, BEAM_PARALYSIS),
@@ -526,7 +513,6 @@ _mon_hex_zap(ZAP_PARALYSE, BEAM_PARALYSIS),
     DCHAR_FIRED_ZAP,
     true,
     false,
-    6
 },
 
 {
@@ -543,7 +529,6 @@ _mon_hex_zap(ZAP_PARALYSE, BEAM_PARALYSIS),
     DCHAR_FIRED_ZAP,
     true,
     false,
-    6
 },
 
 { // Used only by phial of floods
@@ -563,7 +548,6 @@ _mon_hex_zap(ZAP_PARALYSE, BEAM_PARALYSIS),
     DCHAR_WAVY,
     false,
     false,
-    6
 },
 
 _mon_hex_zap(ZAP_CONFUSE, BEAM_CONFUSION),
@@ -583,7 +567,6 @@ _mon_hex_zap(ZAP_TUKIMAS_DANCE, BEAM_TUKIMAS_DANCE, 100),
     NUM_DCHAR_TYPES,
     false,
     false,
-    0
 },
 
 {
@@ -600,7 +583,6 @@ _mon_hex_zap(ZAP_TUKIMAS_DANCE, BEAM_TUKIMAS_DANCE, 100),
     NUM_DCHAR_TYPES,
     true,
     false,
-    4
 },
 
 {
@@ -617,7 +599,6 @@ _mon_hex_zap(ZAP_TUKIMAS_DANCE, BEAM_TUKIMAS_DANCE, 100),
     DCHAR_FIRED_BURST,
     false,
     true,
-    0 // Noise comes from explosion
 },
 
 {
@@ -634,7 +615,22 @@ _mon_hex_zap(ZAP_TUKIMAS_DANCE, BEAM_TUKIMAS_DANCE, 100),
     DCHAR_FIRED_ZAP,
     false,
     true,
-    0 // Noise comes from explosion
+},
+
+{
+    ZAP_BLASTMOTE,
+    "concussive blast",
+    50,
+    new calcdice_calculator<2, 20, 1, 3>,
+    new tohit_calculator<40>,
+    nullptr,
+    nullptr,
+    RED,
+    false,
+    BEAM_FIRE,
+    DCHAR_FIRED_ZAP,
+    false,
+    true,
 },
 
 
@@ -652,7 +648,6 @@ _mon_hex_zap(ZAP_TUKIMAS_DANCE, BEAM_TUKIMAS_DANCE, 100),
     DCHAR_FIRED_ZAP,
     false,
     true,
-    0 // Noise comes from explosion
 },
 
 {
@@ -669,7 +664,6 @@ _mon_hex_zap(ZAP_TUKIMAS_DANCE, BEAM_TUKIMAS_DANCE, 100),
     NUM_DCHAR_TYPES,
     false,
     true,
-    0
 },
 
 {
@@ -686,7 +680,6 @@ _mon_hex_zap(ZAP_TUKIMAS_DANCE, BEAM_TUKIMAS_DANCE, 100),
     NUM_DCHAR_TYPES,
     false,
     false,
-    0
 },
 
 {
@@ -703,7 +696,6 @@ _mon_hex_zap(ZAP_TUKIMAS_DANCE, BEAM_TUKIMAS_DANCE, 100),
     DCHAR_FIRED_ZAP,
     true,
     false,
-    5 // XXX: Maybe louder?
 },
 
 _mon_hex_zap(ZAP_POLYMORPH, BEAM_POLYMORPH),
@@ -722,7 +714,6 @@ _mon_hex_zap(ZAP_POLYMORPH, BEAM_POLYMORPH),
     DCHAR_FIRED_ZAP,
     true,
     false,
-    5 // XXX: Quieter because it's poison?
 },
 
 {
@@ -739,7 +730,22 @@ _mon_hex_zap(ZAP_POLYMORPH, BEAM_POLYMORPH),
     DCHAR_FIRED_ZAP,
     true,
     false,
-    0 // Draining is soundless
+},
+
+{
+    ZAP_KISS_OF_DEATH,
+    "negative energy",
+    25,
+    new calcdice_calculator<2, 11, 3, 5>, // caps at 2d13... spooky
+    new tohit_calculator<AUTOMATIC_HIT>, // XXX: should we let this miss?
+    nullptr,
+    nullptr,
+    DARKGREY,
+    false,
+    BEAM_NEG,
+    DCHAR_FIRED_ZAP,
+    false,
+    false,
 },
 
 {
@@ -756,7 +762,6 @@ _mon_hex_zap(ZAP_POLYMORPH, BEAM_POLYMORPH),
     DCHAR_FIRED_MISSILE,
     false,
     false,
-    8
 },
 
 _mon_hex_zap(ZAP_CHARMING, BEAM_CHARM),
@@ -775,7 +780,6 @@ _mon_hex_zap(ZAP_CHARMING, BEAM_CHARM),
     NUM_DCHAR_TYPES,
     false,
     false,
-    1 // XXX: Should this be soundless?
 },
 
 {
@@ -792,7 +796,6 @@ _mon_hex_zap(ZAP_CHARMING, BEAM_CHARM),
     DCHAR_FIRED_ZAP,
     false,
     false,
-    1
 },
 
 {
@@ -809,7 +812,6 @@ _mon_hex_zap(ZAP_CHARMING, BEAM_CHARM),
     DCHAR_FIRED_ZAP,
     false,
     false,
-    1
 },
 
 {
@@ -826,7 +828,6 @@ _mon_hex_zap(ZAP_CHARMING, BEAM_CHARM),
     NUM_DCHAR_TYPES,
     false,
     false,
-    0
 },
 
 {
@@ -843,7 +844,6 @@ _mon_hex_zap(ZAP_CHARMING, BEAM_CHARM),
     NUM_DCHAR_TYPES,
     false,
     false,
-    0
 },
 
 _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
@@ -862,7 +862,6 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
     DCHAR_FIRED_ZAP,
     false,
     false,
-    1 // XXX: Maybe silent because it's poison?
 },
 
 {
@@ -879,7 +878,6 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
     DCHAR_FIRED_ZAP,
     false,
     true,
-    9 // XXX: Even louder because it's hellish?
 },
 
 {
@@ -896,7 +894,6 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
     DCHAR_FIRED_MISSILE,
     false,
     false,
-    6
 },
 
 {
@@ -913,7 +910,6 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
     DCHAR_FIRED_MISSILE,
     false,
     false,
-    3
 },
 
 {
@@ -930,7 +926,6 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
     DCHAR_FIRED_ZAP,
     true,
     false,
-    1 // XXX: maybe electricity should be louder?
 },
 
 {
@@ -947,7 +942,6 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
     DCHAR_FIRED_ZAP,
     true,
     false,
-    8
 },
 
 {
@@ -964,7 +958,6 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
     DCHAR_FIRED_ZAP,
     false,
     true,
-    6 // XXX: maybe electricity should be louder?
 },
 
 {
@@ -981,7 +974,6 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
     DCHAR_FIRED_ZAP,
     false,
     false,
-    1
 },
 
 {
@@ -998,10 +990,8 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
     DCHAR_FIRED_DEBUG,
     false,
     false,
-    0
 },
 
-// XXX: How loud should breath be?
 {
     ZAP_BREATHE_FIRE,
     "fiery breath",
@@ -1016,7 +1006,6 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
     DCHAR_FIRED_ZAP,
     true,
     false,
-    7
 },
 
 {
@@ -1033,7 +1022,6 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
     DCHAR_FIRED_ZAP,
     true,
     false,
-    6
 },
 
 {
@@ -1050,7 +1038,6 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
     DCHAR_FIRED_ZAP,
     false,
     false,
-    6
 },
 
 {
@@ -1067,7 +1054,6 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
     DCHAR_FIRED_ZAP,
     true,
     false,
-    0 // Explosion does the noise.
 },
 
 {
@@ -1084,7 +1070,6 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
     DCHAR_FIRED_ZAP,
     true,
     false,
-    6
 },
 
 {
@@ -1101,7 +1086,6 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
     DCHAR_FIRED_ZAP,
     false,
     false,
-    6
 },
 
 {
@@ -1118,7 +1102,6 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
     NUM_DCHAR_TYPES,
     false,
     false,
-    0
 },
 
 {
@@ -1135,7 +1118,6 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
     NUM_DCHAR_TYPES,
     false,
     false,
-    0
 },
 
 {
@@ -1152,7 +1134,6 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
     NUM_DCHAR_TYPES,
     false,
     false,
-    0
 },
 
 {
@@ -1169,7 +1150,6 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
     NUM_DCHAR_TYPES,
     true,
     false,
-    6
 },
 
 {
@@ -1186,7 +1166,6 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
     DCHAR_FIRED_ZAP,
     true,
     false,
-    0 // Explosion does the noise.
 },
 
 {
@@ -1203,7 +1182,6 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
     DCHAR_FIRED_MISSILE,
     false,
     false,
-    4
 },
 
 {
@@ -1220,29 +1198,11 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
     NUM_DCHAR_TYPES,
     false,
     false,
-    0
 },
 
 // player spellpower is capped to 50 in spl-zap.cc:spell_zap_power.
 _mon_hex_zap(ZAP_HIBERNATION, BEAM_HIBERNATION),
 
-{
-    ZAP_FLAME_TONGUE,
-    "flame tongue",
-    25,
-    new dicedef_calculator<1, 8, 1, 4>,
-    new tohit_calculator<11, 1, 6>,
-    new dicedef_calculator<3, 3, 1, 12>,
-    new tohit_calculator<7, 1, 6>,
-    RED,
-    false,
-    BEAM_FIRE,
-    DCHAR_FIRED_BOLT,
-    false,
-    false,
-    1
-},
-
 {
     ZAP_SANDBLAST,
     "blast of sand",
@@ -1257,7 +1217,6 @@ _mon_hex_zap(ZAP_HIBERNATION, BEAM_HIBERNATION),
     DCHAR_FIRED_BOLT,
     false,
     false,
-    2 // XXX: Sound 2 for level one spell?
 },
 
 {
@@ -1274,7 +1233,22 @@ _mon_hex_zap(ZAP_HIBERNATION, BEAM_HIBERNATION),
     DCHAR_FIRED_ZAP,
     true,
     false,
-    5
+},
+
+{
+    ZAP_MOMENTUM_STRIKE,
+    "concentrated momentum",
+    50,
+    new calcdice_calculator<3, 24, 1, 6>,
+    new tohit_calculator<9, 1, 12>,
+    nullptr,
+    nullptr,
+    CYAN,
+    false,
+    BEAM_MMISSILE,
+    DCHAR_FIRED_ZAP,
+    false,
+    false,
 },
 
 {
@@ -1291,7 +1265,6 @@ _mon_hex_zap(ZAP_HIBERNATION, BEAM_HIBERNATION),
     DCHAR_FIRED_MISSILE,
     false,
     false,
-    6 // XXX: Less noise because it's poison?
 },
 
 _mon_hex_zap(ZAP_PETRIFY, BEAM_PETRIFY),
@@ -1313,7 +1286,6 @@ _mon_hex_zap(ZAP_SLEEP, BEAM_SLEEP),
                      // hence invisible
     true,
     false,
-    0
 },
 
 {
@@ -1330,7 +1302,6 @@ _mon_hex_zap(ZAP_SLEEP, BEAM_SLEEP),
     DCHAR_FIRED_ZAP,
     true,
     false,
-    0
 },
 
 {
@@ -1347,7 +1318,6 @@ _mon_hex_zap(ZAP_SLEEP, BEAM_SLEEP),
     NUM_DCHAR_TYPES,
     false,
     false,
-    0
 },
 
 {
@@ -1364,7 +1334,6 @@ _mon_hex_zap(ZAP_SLEEP, BEAM_SLEEP),
     DCHAR_FIRED_MISSILE,
     false,
     false,
-    5
 },
 
 {
@@ -1381,7 +1350,6 @@ _mon_hex_zap(ZAP_SLEEP, BEAM_SLEEP),
     DCHAR_FIRED_ZAP,
     true,
     false,
-    2
 },
 
 {
@@ -1398,7 +1366,6 @@ _mon_hex_zap(ZAP_SLEEP, BEAM_SLEEP),
     DCHAR_FIRED_ZAP,
     true,
     false,
-    6
 },
 
 {
@@ -1415,7 +1382,6 @@ _mon_hex_zap(ZAP_SLEEP, BEAM_SLEEP),
     DCHAR_FIRED_ZAP,
     true,
     false,
-    6
 },
 
 {
@@ -1432,7 +1398,6 @@ _mon_hex_zap(ZAP_SLEEP, BEAM_SLEEP),
     DCHAR_FIRED_ZAP,
     true,
     false,
-    6
 },
 
 {
@@ -1449,7 +1414,6 @@ _mon_hex_zap(ZAP_SLEEP, BEAM_SLEEP),
     DCHAR_FIRED_ZAP,
     true,
     false,
-    6
 },
 
 {
@@ -1466,7 +1430,6 @@ _mon_hex_zap(ZAP_SLEEP, BEAM_SLEEP),
     NUM_DCHAR_TYPES,
     false,
     false,
-    0 // dubious
 },
 
 {
@@ -1483,7 +1446,6 @@ _mon_hex_zap(ZAP_SLEEP, BEAM_SLEEP),
     DCHAR_FIRED_MISSILE,
     false,
     true,
-    0 // Noise comes from explosion
 },
 
 {
@@ -1500,7 +1462,6 @@ _mon_hex_zap(ZAP_SLEEP, BEAM_SLEEP),
     DCHAR_FIRED_MISSILE,
     false,
     false,
-    1
 },
 
 {
@@ -1517,7 +1478,6 @@ _mon_hex_zap(ZAP_SLEEP, BEAM_SLEEP),
     DCHAR_FIRED_ZAP,
     true,
     false,
-    10,
 },
 
 {
@@ -1534,7 +1494,6 @@ _mon_hex_zap(ZAP_SLEEP, BEAM_SLEEP),
     DCHAR_FIRED_ZAP,
     true,
     false,
-    0
 },
 
 {
@@ -1551,7 +1510,6 @@ _mon_hex_zap(ZAP_SLEEP, BEAM_SLEEP),
     DCHAR_FIRED_ZAP,
     true,
     false,
-    0
 },
 
 {
@@ -1568,7 +1526,6 @@ _mon_hex_zap(ZAP_SLEEP, BEAM_SLEEP),
     DCHAR_FIRED_MISSILE,
     false,
     false,
-    1
 },
 
 _mon_hex_zap(ZAP_DIMENSION_ANCHOR, BEAM_DIMENSION_ANCHOR),
@@ -1592,7 +1549,6 @@ _mon_hex_zap(ZAP_DRAIN_MAGIC, BEAM_DRAIN_MAGIC),
     DCHAR_FIRED_ZAP,
     false,
     false,
-    0
 },
 
 {
@@ -1609,7 +1565,6 @@ _mon_hex_zap(ZAP_DRAIN_MAGIC, BEAM_DRAIN_MAGIC),
     DCHAR_FIRED_ZAP,
     false,
     true,
-    0 // handled by explosion
 },
 
 {
@@ -1626,7 +1581,6 @@ _mon_hex_zap(ZAP_DRAIN_MAGIC, BEAM_DRAIN_MAGIC),
     NUM_DCHAR_TYPES,
     false,
     false,
-    0
 },
 
 {
@@ -1643,7 +1597,6 @@ _mon_hex_zap(ZAP_DRAIN_MAGIC, BEAM_DRAIN_MAGIC),
     DCHAR_FIRED_BURST,
     true,
     false,
-    5
 },
 
 {
@@ -1660,7 +1613,6 @@ _mon_hex_zap(ZAP_DRAIN_MAGIC, BEAM_DRAIN_MAGIC),
     NUM_DCHAR_TYPES,
     false,
     false,
-    0 // No additional effect noise.
 },
 
 {
@@ -1677,7 +1629,6 @@ _mon_hex_zap(ZAP_DRAIN_MAGIC, BEAM_DRAIN_MAGIC),
     NUM_DCHAR_TYPES,
     false,
     false,
-    0 // No additional effect noise.
 },
 
 {
@@ -1694,7 +1645,6 @@ _mon_hex_zap(ZAP_DRAIN_MAGIC, BEAM_DRAIN_MAGIC),
     NUM_DCHAR_TYPES,
     false,
     false,
-    0 // No additional effect noise.
 },
 
 {
@@ -1711,7 +1661,6 @@ _mon_hex_zap(ZAP_DRAIN_MAGIC, BEAM_DRAIN_MAGIC),
     DCHAR_FIRED_MISSILE,
     false,
     false,
-    6
 },
 
 {
@@ -1728,7 +1677,6 @@ _mon_hex_zap(ZAP_DRAIN_MAGIC, BEAM_DRAIN_MAGIC),
     DCHAR_FIRED_BOLT,
     false,
     false,
-    1
 },
 
 {
@@ -1745,7 +1693,6 @@ _mon_hex_zap(ZAP_DRAIN_MAGIC, BEAM_DRAIN_MAGIC),
     DCHAR_FIRED_BURST,
     false,
     true,
-    0 // Noise comes from explosion
 },
 
 {
@@ -1753,7 +1700,7 @@ _mon_hex_zap(ZAP_DRAIN_MAGIC, BEAM_DRAIN_MAGIC),
     "hail",
     100,
     new calcdice_calculator<3, 10, 1, 2>,
-    new tohit_calculator<AUTOMATIC_HIT>,
+    new tohit_calculator<9, 1, 10>,
     nullptr,
     nullptr,
     ETC_ICE,
@@ -1762,7 +1709,6 @@ _mon_hex_zap(ZAP_DRAIN_MAGIC, BEAM_DRAIN_MAGIC),
     DCHAR_FIRED_BURST,
     false,
     false,
-    0
 },
 
 {
@@ -1779,7 +1725,6 @@ _mon_hex_zap(ZAP_DRAIN_MAGIC, BEAM_DRAIN_MAGIC),
     DCHAR_FIRED_ZAP,
     true,
     false,
-    10
 },
 
 {
@@ -1796,7 +1741,6 @@ _mon_hex_zap(ZAP_DRAIN_MAGIC, BEAM_DRAIN_MAGIC),
     DCHAR_FIRED_ZAP,
     true,
     false,
-    0
 },
 
 {
@@ -1813,7 +1757,6 @@ _mon_hex_zap(ZAP_DRAIN_MAGIC, BEAM_DRAIN_MAGIC),
     NUM_DCHAR_TYPES,
     false,
     false,
-    0
 },
 
 {
@@ -1830,7 +1773,6 @@ _mon_hex_zap(ZAP_DRAIN_MAGIC, BEAM_DRAIN_MAGIC),
     NUM_DCHAR_TYPES,
     false,
     false,
-    0
 },
 
 {
@@ -1847,7 +1789,22 @@ _mon_hex_zap(ZAP_DRAIN_MAGIC, BEAM_DRAIN_MAGIC),
     NUM_DCHAR_TYPES,
     false,
     false,
-    1 // XXX: Should this be soundless?
+},
+
+{
+    ZAP_PLASMA,
+    "fiery plasma",
+    200,
+    new dicedef_calculator<1, 11, 3, 5>,
+    new tohit_calculator<7, 1, 40>,
+    new dicedef_calculator<3, 10, 1, 17>,
+    new tohit_calculator<16, 1, 40>,
+    RED,
+    false,
+    BEAM_FIRE,
+    DCHAR_FIRED_ZAP,
+    true,
+    false
 },
 
 };
